@@ -878,8 +878,11 @@ extern void GGobi_update_data(datad *, ggobid *gg);
  Get a list of the human readable names of the different 
  input modes for data  sources. These include
  ASCII, XML, database, URL, etc.
+
+ Caller is responsible for freeing the array, but not its
+ elements!
  */
-const gchar *const * GGobi_getDataModeNames(int *n);
+const gchar ** GGobi_getDataModeNames(int *n);
 
 
 /** 
@@ -978,12 +981,17 @@ ggobid *create_ggobi(InputDescription *desc);
 
 /**
  */
-datad* datad_create(int nr, int nc, ggobid *gg);
+GGOBI_EXPORT datad* datad_create(int nr, int nc, ggobid *gg);
 
 
 void GGobi_setSessionOptions(GGobiOptions *opts);
 
 
+/**
+Set the fatal error handler so that host applications
+can control how to recover from GGobi "exceptions"
+ */
+void GGOBI(setErrorHandlers)(FatalErrorHandler err);
 
 /**
 
