@@ -116,9 +116,9 @@ void       brush_draw_brush (splotd *, GdkDrawable *, datad *, ggobid *);
 void       brush_draw_label (splotd *, GdkDrawable *, datad *, ggobid *);
 void       brush_event_handlers_toggle (splotd *, gboolean);
 void       brush_init (datad *, ggobid *);
-void       brush_motion (icoords *, gboolean, gboolean, cpaneld *, splotd *, ggobid *);
+gboolean   brush_motion (icoords *, gboolean, gboolean, cpaneld *, splotd *, ggobid *);
 gboolean   brush_once (gboolean force, splotd *, ggobid *);
-void       brush_once_and_redraw (gboolean binningp, splotd *sp, displayd *display, ggobid *gg);
+gboolean   brush_once_and_redraw (gboolean binningp, splotd *sp, displayd *display, ggobid *gg);
 void       brush_options_cb (gpointer, guint, GtkCheckMenuItem *);
 void       brush_pos_init (splotd *);
 void       brush_prev_vectors_update (datad *, ggobid *);
@@ -145,30 +145,30 @@ gchar*     computeTitle (gboolean, displayd *, ggobid *);
 void       copy_mat(gdouble **, gdouble **, gint, gint);
 void       cpanel_brush_init (cpaneld *, ggobid *);
 void       cpanel_brush_make (ggobid *);
-void       cpanel_brush_set (cpaneld *, ggobid *);
+void       cpanel_brush_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_ctour_make (ggobid *);
 void       cpanel_edgeedit_init (cpaneld *, ggobid *);
 void       cpanel_edgeedit_make (ggobid *);
-void       cpanel_edgeedit_set (cpaneld *, ggobid *);
+void       cpanel_edgeedit_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_identify_init (cpaneld *, ggobid *);
 void       cpanel_identify_make (ggobid *);
-void       cpanel_identify_set (cpaneld *, ggobid *);
+void       cpanel_identify_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_movepts_make (ggobid *);
 void       cpanel_p1d_init (cpaneld *, ggobid *);
-void       cpanel_p1d_set (cpaneld *, ggobid *);
+void       cpanel_p1d_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_p1dplot_make (ggobid *);
 void       cpanel_parcoords_init (cpaneld *, ggobid *);
 GtkWidget *cpanel_parcoords_make (ggobid *);
-void       cpanel_parcoords_set (cpaneld *, GtkWidget *panel, ggobid *);
+void       cpanel_parcoords_set (displayd *, cpaneld *, GtkWidget *panel, ggobid *);
 void       cpanel_rotation_init (cpaneld *, ggobid *);
 void       cpanel_rotation_make (ggobid *);
-void       cpanel_rotation_set (cpaneld *, ggobid *);
+void       cpanel_rotation_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_scale_init (cpaneld *, ggobid *);
 void       cpanel_scale_make (ggobid *);
-void       cpanel_scale_set (cpaneld *, ggobid *);
+void       cpanel_scale_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_scatmat_init (cpaneld *, ggobid *);
 void       cpanel_scatmat_make (ggobid *);
-void       cpanel_scatmat_set (cpaneld *, ggobid *);
+void       cpanel_scatmat_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_set (displayd *, ggobid *);
 void       cpanel_show_cb (GtkCheckMenuItem *w, guint action);
 void       cpanel_t1d_init (cpaneld *, ggobid *);
@@ -176,15 +176,15 @@ void       cpanel_t2d3_init (cpaneld *, ggobid *);
 void       cpanel_t2d_init (cpaneld *, ggobid *);
 void       cpanel_tcorr_init (cpaneld *, ggobid *);
 void       cpanel_tour1d_make (ggobid *);
-void       cpanel_tour1d_set (cpaneld *, ggobid *);
+void       cpanel_tour1d_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_tour2d3_make (ggobid *);
-void       cpanel_tour2d3_set (cpaneld *, ggobid *);
+void       cpanel_tour2d3_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_tour2d_make (ggobid *);
-void       cpanel_tour2d_set (cpaneld *, ggobid *);
-void       cpanel_tourcorr_set (cpaneld *, ggobid *);
+void       cpanel_tour2d_set (displayd *, cpaneld *, ggobid *);
+void       cpanel_tourcorr_set (displayd *, cpaneld *, ggobid *);
 void       cpanel_xyplot_init (cpaneld *, ggobid *);
 void       cpanel_xyplot_make (ggobid *);
-void       cpanel_xyplot_set (cpaneld *, ggobid *);
+void       cpanel_xyplot_set (displayd *, cpaneld *, ggobid *);
 displayd * createDisplayFromDescription (ggobid *, GGobiDisplayDescription *desc);
 GtkWidget* create_variable_notebook (GtkWidget *box, GtkSelectionMode mode, vartyped vtype, datatyped dtype, GtkSignalFunc func, ggobid *);
 void       ctour_event_handlers_toggle (splotd *, gboolean);
@@ -237,9 +237,10 @@ void       display_window_init (windowDisplayd *, gint, ggobid *);
 void       display_write_svg (ggobid *);
 void       displays_plot (splotd *, RedrawStyle, ggobid *);
 void       displays_tailpipe (RedrawStyle, ggobid *);
+void       set_display_options(displayd *display, ggobid *gg);
 gint       do_ash1d (gfloat *, gint, gint, gint, gfloat *, gfloat *, gfloat *, gfloat *);
 void       do_last_increment (vector_f, vector_f, gfloat, gint);
-void       draw_3drectangle (GdkDrawable *drawable, gint x, gint y, gint width, gint height, ggobid *gg);
+void       draw_3drectangle (GtkWidget *w, GdkDrawable *drawable, gint x, gint y, gint width, gint height, ggobid *gg);
 void       draw_glyph (GdkDrawable *, glyphd *, icoords *, gint, ggobid *);
 gint       dsvd (gdouble **a, gint m, gint n, gfloat *w, gdouble **v);
 PipelineMode projection_get (ggobid *);
@@ -253,13 +254,14 @@ void       edges_free (datad *, ggobid *);
 gboolean   edges_read (InputDescription *desc, gboolean, datad *, ggobid *);
 gboolean   edgeset_add (displayd *);
 void       edgeset_add_cb (GtkWidget *w, datad *e);
+gint       edgesets_count (ggobid *gg);
 void       eigenvals_get (gfloat *, datad *);
 gboolean   exclude_link_by_id (gint k, datad *source_d, ggobid *gg);
 gint       fcompare (const void *x1, const void *x2);
 void       filename_get_r (ggobid *, guint, GtkWidget *);
 void       filename_get_w (GtkWidget *, ggobid *);
-gboolean   fileset_read (const gchar *, ggobid *);
-gboolean   fileset_read_init (const gchar *ldata_in, ggobid *);
+gboolean   fileset_read (const gchar *, const gchar *modeName, GGobiPluginInfo *, ggobid *);
+gboolean   fileset_read_init (const gchar *ldata_in, const gchar *, GGobiPluginInfo *, ggobid *);
 void       find_glyph_type_and_size (gint, glyphd *);
 gint       find_keepers (gint ncols_current, gint nc, gint *cols, gint *keepers);
 gint       find_nearest_edge (splotd *sp, displayd *display, ggobid *gg);
@@ -286,12 +288,13 @@ gboolean   hidden_read (InputDescription *desc, gboolean, datad *, ggobid *);
 void       identify_event_handlers_toggle (splotd *, gboolean);
 gchar *    identify_label_fetch (gint k, cpaneld *cpanel, datad *d, ggobid *);
 void       identify_link_by_id (gint k, datad *source_d, ggobid *gg);
-gboolean   impute_fixed (gint, gint nvars, gint *vars, datad *, ggobid *);
+gboolean   impute_fixed (ImputeType, gfloat val, int nvars, gint *vars, datad *, ggobid *);
+gboolean   impute_mean_or_median (gint, gint, gint *, datad *, ggobid *);
 void       impute_random (datad *, gint nvars, gint *vars, ggobid *);
 void       impute_window_open (ggobid *);
 gint       include_hiddens (gboolean include, datad *d, ggobid *gg);
 void       increment_tour(vector_f, vector_f, gfloat, gfloat, gfloat *, gint);
-void       initSessionOptions(void);
+void       initSessionOptions(int argc, char **argv);
 void       init_plot_GC (GdkWindow *, ggobid *);
 void       init_var_GCs (GtkWidget *, ggobid *);
 gdouble    inner_prod (gdouble *, gdouble *, gint);
@@ -322,7 +325,6 @@ void       movepts_history_add (gint id, splotd *sp, datad *, ggobid *);
 void       movepts_history_delete_last (datad *, ggobid *);
 gdouble    myrint (gdouble x);
 gint       ndatad_with_vars_get (ggobid *gg);
-void       newvar_add (gint vtype, gchar *vname, datad *d, ggobid *gg);
 void       newvar_add_with_values (gdouble *, gint, gchar *, vartyped type, gint nlevels, gchar **level_names, gint *level_values, gint *level_counts, datad *, ggobid *);
 void       norm (gdouble *, gint);
 gint       option_menu_index (GtkOptionMenu *);
@@ -336,7 +338,8 @@ void       pan_step (splotd *, gint, ggobid *);
 void       pan_step_key (splotd *sp, guint keyval, ggobid *gg);
 void       parcoords_cpanel_init (cpaneld*, ggobid *);
 void       parcoords_mode_menu_make (GtkAccelGroup *, GtkSignalFunc, ggobid *, gboolean);
-displayd*  parcoords_new (gboolean, gint, gint *, datad *, ggobid *);
+displayd*  parcoords_new_with_vars (gboolean, gint, gint *, datad *, ggobid *);
+displayd*  parcoords_new (displayd *dpy, gboolean, gint, gint *, datad *, ggobid *);
 void       parcoords_reset_arrangement (displayd *, gint, ggobid *);
 gboolean   parcoords_varsel (cpaneld *, splotd *, gint, gint *, ggobid *);
 gint       parse_command_line (gint *argc, gchar **av);
@@ -384,7 +387,7 @@ void       scale_set_default_values (GtkScale *scale);
 void       scaling_visual_cues_draw (splotd *, GdkDrawable *, ggobid *);
 void       scatmat_cpanel_init (cpaneld *, ggobid *);
 void       scatmat_mode_menu_make (GtkAccelGroup *, GtkSignalFunc, ggobid *gg, gboolean);
-displayd*  scatmat_new (gboolean, gint, gint *, gint, gint *, datad *, ggobid *);
+displayd*  scatmat_new (displayd *, gboolean, gint, gint *, gint, gint *, datad *, ggobid *);
 gboolean   scatmat_varsel (GtkWidget *, cpaneld *, splotd *, gint, gint, gint *, gint, gboolean, ggobid *);
 gboolean   scatmat_varsel_simple (cpaneld *, splotd *, gint, gint *, ggobid *);
 void       scatterplot_cpanel_init (cpaneld *, PipelineMode, ggobid *);
@@ -613,7 +616,7 @@ void       vartable_limits_set_by_var (gint j, datad *d);
 void       vartable_open (ggobid *);
 void       vartable_realloc (gint, datad *);
 void       vartable_row_append (gint j, datad *, ggobid *);
-void       vartable_show_page (displayd*, ggobid*);
+void       vartable_show_page (datad*, ggobid*);
 void       vartable_stats_set (datad *);
 void       vartable_stats_set_by_var (gint j, datad *);
 void       vectorb_alloc (vector_b *, gint);
@@ -678,11 +681,12 @@ void       zoom_step (splotd *, gint, gint, rectd *, ggobid*);
 
 /*tsplot functions*/
 GtkWidget* cpanel_tsplot_make (ggobid *);
-void      cpanel_tsplot_set (cpaneld *, GtkWidget *, ggobid *);
+void      cpanel_tsplot_set (displayd *, cpaneld *, GtkWidget *, ggobid *);
 void      tsplot_cpanel_init (cpaneld*, ggobid *);
 void      tsplot_menus_make (ggobid *gg);
 void      tsplot_mode_menu_make (GtkAccelGroup *, GtkSignalFunc, ggobid *gg, gboolean);
-displayd* tsplot_new (gboolean, gint, gint *, datad *, ggobid *);
+displayd* tsplot_new (displayd *, gboolean, gint, gint *, datad *, ggobid *);
+displayd *tsplot_new_with_vars (gboolean missing_p, gint nvars, gint *vars, datad *d, ggobid *gg) ;
 void      tsplot_reset_arrangement (displayd *, gint, ggobid *);
 gboolean  tsplot_varsel (GtkWidget *, displayd *display, splotd *sp, gint jvar, gint toggle, gint mouse, cpaneld *cpanel,  ggobid *gg);
 void      tsplot_whiskers_make (splotd *, displayd *, ggobid *);
@@ -695,7 +699,7 @@ gint      ggobi_remove (ggobid *);
 void      subset_init (datad *d, ggobid *gg);
 
 
-
+displayd *createBarchart(displayd *display, gboolean missing_p, splotd * sp, gint var, datad * d,  ggobid * gg);
 void barchart_scaling_visual_cues_draw (splotd *sp, GdkDrawable *drawable, ggobid *gg);
 gboolean barchart_active_paint_points (splotd *sp, datad *, ggobid *); 
 void barchart_add_bar_cues (splotd *sp, GdkDrawable *drawable, ggobid *gg);
@@ -713,7 +717,7 @@ void barchart_recalc_group_dimensions (barchartSPlotd *sp, ggobid *gg);
 gboolean barchart_redraw (splotd *sp, datad *d, ggobid *gg, gboolean binned);
 void barchart_splot_add_plot_labels (splotd *, GdkDrawable *, ggobid *);
 GtkWidget* cpanel_barchart_make (ggobid *gg);
-void cpanel_barchart_set (cpaneld *cpanel, GtkWidget *panel, ggobid *gg);
+void cpanel_barchart_set (displayd *, cpaneld *, GtkWidget *panel, ggobid *gg);
 
 void barchart_scale_event_handlers_toggle(splotd *sp, gboolean state);
 
@@ -758,10 +762,28 @@ GGobiPluginInfo *readPluginFile(const char * const fileName, GGobiInitInfo *info
 gboolean registerPlugin(ggobid *gg, GGobiPluginInfo *plugin);
 
 
+GGobiOptions *GGOBI_getSessionOptions();
+
+
+
 #ifdef WIN32
 /* needed by transform.c */
 extern double erf(double x);
 extern double erfc(double x);
 #endif
 
+gboolean read_xml(InputDescription *desc, ggobid *gg, GGobiPluginInfo *plugin);
+gboolean read_csv(InputDescription *desc, ggobid *gg, GGobiPluginInfo *plugin);
+gboolean read_ascii(InputDescription *desc, ggobid *gg, GGobiPluginInfo *plugin);
+
+InputDescription *read_xml_input_description(const char * const fileName, const char * const modeName, 
+					     ggobid *gg, GGobiPluginInfo *info);
+InputDescription *read_csv_input_description(const char * const fileName, const char * const modeName, 
+					     ggobid *gg, GGobiPluginInfo *info);
+InputDescription *read_ascii_input_description(const char * const fileName, const char * const modeName, 
+					       ggobid *gg, GGobiPluginInfo *info);
+
+
+void resetDataMode();
+
 #endif
