@@ -123,13 +123,38 @@ hasPathToCenter (noded *n, noded *referringnode, datad *d, datad *e,
 /*                   callbacks                                     */
 /*-----------------------------------------------------------------*/
 
-void radial_cb (GtkButton *button, PluginInstance *inst)
+void radial_auto_update_cb (GtkToggleButton *btn, PluginInstance *inst)
+{
+  glayoutd *gl = glayoutFromInst (inst);
+
+  gl->radialAutoUpdate = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(btn));
+}
+
+void radial_new_data_cb (GtkToggleButton *btn, PluginInstance *inst)
+{
+  glayoutd *gl = glayoutFromInst (inst);
+
+  gl->radialNewData = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(btn));
+}
+
+
+void do_radial(glayoutd *gl, datad *d, datad *e, displayd *dsp, ggobid *gg);
+void 
+radial_cb (GtkButton *button, PluginInstance *inst)
 {
   ggobid *gg = inst->gg;
   glayoutd *gl = glayoutFromInst (inst);
   displayd *dsp = gg->current_display;
   datad *d = gl->dsrc;
   datad *e = gl->e;
+
+  do_radial(gl, d, e, dsp, gg);
+}
+
+
+void 
+do_radial(glayoutd *gl, datad *d, datad *e, displayd *dsp, ggobid *gg)
+{
   glong *visible;
   gint nvisible;
 /*-- to add variables --*/
@@ -145,8 +170,9 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
   gdouble *values;
   gchar **rownames, **colnames, **rowids;
   displayd *dspnew;
-  gboolean edges_displayed;
   gboolean redisplay_all = false;  /* if points are hidden in this function */
+  DisplayOptions *opts = NULL;
+  GtkWidget *w;
 
   if (e == NULL) {
     g_printerr ("Unable to proceed: no edge set is specified\n");
@@ -163,7 +189,6 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
     return;
   }
 
-
 /*-- This may not belong here, but where exactly?  As soon as the
      panel is opened  --*/
 /*
@@ -175,11 +200,6 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
 
   visible = (glong *) g_malloc (d->nrows_in_plot * sizeof (glong));
   nvisible = visible_set (visible, d);
-/*
-  init = (gl->radial == NULL ||
-          d != gl->radial->d ||
-          d->nrows_in_plot != nvisible);
-*/
   initRadialLayout (visible, nvisible, gg, gl);
 
 /*
@@ -300,53 +320,20 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
    * have no variables, we don't want creating a datad to trigger
    * the initialization of this plot.   This takes care of it.
   */
-  sessionOptions->info->createInitialScatterPlot = false;
+  GGOBI_getSessionOptions()->info->createInitialScatterPlot = false;
   /*-- --*/
 
-  dnew = datad_create (nvisible, nc, gg);
-  dnew->name = g_strdup ("radial");
-  dnew->nickname = g_strdup ("rad");
-
-  GGOBI(setData) (values, rownames, colnames, nvisible, nc, dnew, false,
-    gg, rowids, false, desc);
-
-  /*-- copy the color and glyph vectors from d to dnew --*/
-  for (i=0; i<nvisible; i++) {
-    dnew->color.els[i] = dnew->color_now.els[i] = dnew->color_prev.els[i] =
-      d->color.els[visible[i]];
-    dnew->glyph.els[i].type = dnew->glyph_now.els[i].type =
-      dnew->glyph_prev.els[i].type = d->glyph.els[visible[i]].type;
-    dnew->glyph.els[i].size = dnew->glyph_now.els[i].size =
-      dnew->glyph_prev.els[i].size = d->glyph.els[visible[i]].size;
-  }
-
-/*
- * open a new scatterplot with the new data, and display edges
- * as they're displayed in the current datad.
-*/
-  DefaultDisplayOptions.axes_show_p = false;
-  DefaultDisplayOptions.edges_undirected_show_p = true;
-  dspnew = GGOBI(newScatterplot) (0, 1, dnew, gg);
-  DefaultDisplayOptions.axes_show_p = true;  /*-- restore it --*/
-  DefaultDisplayOptions.edges_undirected_show_p = false;
-
-  setDisplayEdge (dspnew, e);
-
-  displays_tailpipe (FULL, gg);
-
-  g_free (values);
-  g_free (rownames);
-  g_free (colnames);
-/*
-  for (m=0; m<nvisible; m++)
-    g_free (rowids[m]);
-  g_free (rowids);
-*/
-  g_free (visible);
-
-/*-- overwrite new radial variables --*/
-/*
+  /*
+   * Determine whether to create a new datad or whether to
+   * re-use an existing datad.  For now, only re-use a datad
+   * and a display if current_display->d->nrows = nvisible
+   */
+  if (gl->radialNewData == false && gl->d != NULL &&
+      gl->d->nrows == nvisible && gl->d->ncols == 8)
+  {
     gint j;
+    datad *d = gl->d;
+
     j = GGOBI(getVariableIndex)("x", d, gg);
     for (i=0; i<d->nrows; i++)
       d->raw.vals[i][j] = d->tform.vals[i][j] = x[i];
@@ -357,6 +344,11 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
       d->raw.vals[i][j] = d->tform.vals[i][j] = y[i];
     limits_set_by_var (j, true, true, d, gg);
 
+    j = GGOBI(getVariableIndex)("depth", d, gg);
+    for (i=0; i<d->nrows; i++)
+      d->raw.vals[i][j] = d->tform.vals[i][j] = y[i];
+    limits_set_by_var (j, true, true, d, gg);
+
     j = GGOBI(getVariableIndex)("in degree", d, gg);
     for (i=0; i<d->nrows; i++)
       d->raw.vals[i][j] = d->tform.vals[i][j] = inDegree[i];
@@ -367,25 +359,74 @@ void radial_cb (GtkButton *button, PluginInstance *inst)
       d->raw.vals[i][j] = d->tform.vals[i][j] = outDegree[i];
     limits_set_by_var (j, true, true, d, gg);
 
-    j = GGOBI(getVariableIndex)("nparents", d, gg);
+    j = GGOBI(getVariableIndex)("nParents", d, gg);
     for (i=0; i<d->nrows; i++)
       d->raw.vals[i][j] = d->tform.vals[i][j] = nParents[i];
     limits_set_by_var (j, true, true, d, gg);
 
-    j = GGOBI(getVariableIndex)("nchildren", d, gg);
+    j = GGOBI(getVariableIndex)("nChildren", d, gg);
     for (i=0; i<d->nrows; i++)
       d->raw.vals[i][j] = d->tform.vals[i][j] = nChildren[i];
     limits_set_by_var (j, true, true, d, gg);
 
-    j = GGOBI(getVariableIndex)("nsiblings", d, gg);
+    j = GGOBI(getVariableIndex)("nSiblings", d, gg);
     for (i=0; i<d->nrows; i++)
       d->raw.vals[i][j] = d->tform.vals[i][j] = nSiblings[i];
     limits_set_by_var (j, true, true, d, gg);
 
     tform_to_world (d, gg);
-    displays_tailpipe (FULL, gg);
   }
+  else {
+    gl->d = dnew = datad_create (nvisible, nc, gg);
+    dnew->name = g_strdup_printf ("rad:%s", 
+      (gchar *) g_array_index (d->rowlab, gchar *, gl->centerNodeIndex));
+    dnew->nickname = g_strdup ("rad");
+
+    GGOBI(setData) (values, rownames, colnames, nvisible, nc, dnew, false,
+		    gg, rowids, false, desc);
+
+    /*-- copy the color and glyph vectors from d to dnew --*/
+    for (i=0; i<nvisible; i++) {
+      dnew->color.els[i] = dnew->color_now.els[i] = dnew->color_prev.els[i] =
+	d->color.els[visible[i]];
+      dnew->glyph.els[i].type = dnew->glyph_now.els[i].type =
+	dnew->glyph_prev.els[i].type = d->glyph.els[visible[i]].type;
+      dnew->glyph.els[i].size = dnew->glyph_now.els[i].size =
+	dnew->glyph_prev.els[i].size = d->glyph.els[visible[i]].size;
+    }
+
+    /*
+     * open a new scatterplot with the new data, and display edges
+     * as they're displayed in the current datad.
+     */
+    opts = GGOBI(getDefaultDisplayOptions)();
+    opts->axes_show_p = false;
+    opts->edges_undirected_show_p = true;
+    dspnew = GGOBI(newScatterplot) (0, 1, dnew, gg);
+    opts->axes_show_p = true;  /*-- restore it --*/
+    opts->edges_undirected_show_p = false;
+
+    setDisplayEdge (dspnew, e);
+  }
+
+  displays_tailpipe (FULL, gg);
+
+  /* enable the check button widget for choosing to re-use a datad and
+     displays */
+  if (gl && gl->window) {
+    w = widget_find_by_name (gl->window, "RADIAL:newdata");
+    gtk_widget_set_sensitive (w, true);
+  }
+
+  g_free (values);
+  g_free (rownames);
+  g_free (colnames);
+/*
+  for (m=0; m<nvisible; m++)
+    g_free (rowids[m]);
+  g_free (rowids);
 */
+  g_free (visible);
 
   g_free (x);
   g_free (y);
@@ -401,14 +442,36 @@ void radial_center_set_cb (ggobid *gg, gint index,
   gint state, datad *d, PluginInstance *inst)
 {
   glayoutd *gl = glayoutFromInst (inst);
+  GtkWidget *entry;
 
-  GtkWidget *entry = (GtkWidget *) gtk_object_get_data (GTK_OBJECT(gl->window),
+  if (gl == NULL || gl->window == NULL)
+    return;
+
+  entry = (GtkWidget *) gtk_object_get_data (GTK_OBJECT(gl->window),
     "CENTERNODE");
+  if (entry == NULL)
+    return;
 
   if (state == STICKY && index >= 0) {
+    /*-- remove all sticky labels but the last one --*/
+    gint i, k, nsticky = g_slist_length (d->sticky_ids);
+    for (i = 0; i<nsticky-1; i++) {
+      k = (gint) g_slist_nth_data (d->sticky_ids, 0);
+      d->sticky_ids = g_slist_remove (d->sticky_ids, 
+				      g_slist_nth_data (d->sticky_ids, 0));
+      sticky_id_link_by_id (STICKY_REMOVE, k, d, gg);
+      gtk_signal_emit(GTK_OBJECT(gg),
+        getGGobiSignal(STICKY_POINT_REMOVED_SIGNAL), 0,
+        (gint) UNSTICKY, d);
+    }
+
     gtk_entry_set_text (GTK_ENTRY (entry),
       (gchar *) g_array_index (d->rowlab, gchar *, index));
     gl->centerNodeIndex = index;
+
+    if (gl->radialAutoUpdate) {
+       do_radial(gl, gl->dsrc, gl->e, gg->current_display, gg);
+    }
   }
 }
 
@@ -458,10 +521,6 @@ void radial_highlight_sticky_edges (ggobid *gg, gint index, gint state,
       e->glyph.els[k].size = e->glyph_now.els[k].size = gg->glyph_id.size;
       e->glyph.els[k].type = e->glyph_now.els[k].type = gg->glyph_id.type;
     } else {
-/*
-      gint a = d->rowid.idv.els[e->edge.endpoints[k].a];
-      gint b = d->rowid.idv.els[e->edge.endpoints[k].b];
-*/
       gint a, b;
       endpointsd *endpoints;
       endpoints = resolveEdgePoints(e, d);
@@ -828,3 +887,89 @@ setNodePositions (glayoutd *gl, datad *d) {
   setChildNodePositions (gl->radial->centerNode, gl, d);
 }
 
+void
+do_radial_plugin (PluginInstance *inst, gint center, datad *d, datad *e, displayd *dsp, ggobid *gg)
+{
+   glayoutd *gl;
+   gl = glayoutFromInst (inst);
+
+   if(!gl) {
+    gl = (glayoutd *) g_malloc (sizeof (glayoutd));
+    glayout_init (gl);
+    inst->data = gl;
+   } 
+
+   gl->dsrc = d;
+   gl->e = e;
+   gl->centerNodeIndex = center;
+
+   do_radial(gl, d, e, dsp, gg);
+}
+
+#ifdef WITH_R
+#ifdef NEW
+#undef NEW
+#endif
+#include "RSCommon.h"
+
+USER_OBJECT_
+S_radial_cb(USER_OBJECT_ plugin, USER_OBJECT_ centerNode, USER_OBJECT_ data, 
+              USER_OBJECT_ edges, USER_OBJECT_ display)
+{
+   datad *d, *e;
+   glayoutd *gl;
+   ggobid *gg;
+   PluginInstance *inst;
+   displayd *dsp;
+
+   inst = (PluginInstance *) R_ExternalPtrAddr(plugin);
+
+   gl = glayoutFromInst (inst);
+
+   if(!gl) {
+    gl = (glayoutd *) g_malloc (sizeof (glayoutd));
+    glayout_init (gl);
+    inst->data = gl;
+   }
+
+   gl->centerNodeIndex = INTEGER_DATA(centerNode)[0];
+
+   gg = inst->gg;
+   gl->dsrc = d = (datad *) R_ExternalPtrAddr(data);
+   gl->e = e = (datad *) R_ExternalPtrAddr(edges);
+
+   dsp = (displayd *) R_ExternalPtrAddr(data);
+
+   do_radial(gl, d, e, dsp, gg);
+
+     /* Return the new display. */
+   return(NULL_USER_OBJECT);
+}
+#endif
+
+/*-- API routines --*/
+gint radial_auto_update_set (gboolean state, PluginInstance *inst)
+{
+  glayoutd *gl = glayoutFromInst (inst);
+  GtkWidget *w;
+
+  if (gl) {
+    w = widget_find_by_name (gl->window, "RADIAL:autoupdate");
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), state);
+    return 1;
+  }
+  return 0;
+}
+gint radial_new_data_set (gboolean state, PluginInstance *inst)
+{
+  glayoutd *gl = glayoutFromInst (inst);
+  GtkWidget *w;
+
+  if (gl) {
+    w = widget_find_by_name (gl->window, "RADIAL:newdata");
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), state);
+    return 1;
+  }
+  return 0;
+}
+/*  */
