@@ -4,6 +4,7 @@
 #include "GGobiAPI.h"
 
 #include <stdio.h>
+#include <string.h>
 #include <math.h>
 
 #include "plugin.h"
@@ -30,8 +31,10 @@ ggv_scramble (ggvisd *ggv, ggobid *gg)
 }
 
 void
-ggv_datad_create (datad *dsrc, datad *e, displayd *dsp, ggvisd *ggv, ggobid *gg)
+ggv_datad_create (datad *dsrc, datad *e, displayd *dsp, PluginInstance *inst)
 {
+  ggvisd *ggv = ggvisFromInst (inst); 
+  ggobid *gg = inst->gg;
   gint i, j;
   gint nc = ggv->dim;   /*-- default:  3d --*/
   gchar **rownames, **colnames, **rowids;
@@ -96,7 +99,9 @@ ggv_datad_create (datad *dsrc, datad *e, displayd *dsp, ggvisd *ggv, ggobid *gg)
    * have no variables, we don't want creating a datad to trigger
    * the initialization of this plot.   This takes care of it.
   */
-  sessionOptions->info->createInitialScatterPlot = false;
+
+  GGOBI_getSessionOptions()->info->createInitialScatterPlot = false;
+
   /*-- --*/
 
   dnew = datad_create (dsrc->nrows, nc, gg);
@@ -125,6 +130,7 @@ ggv_datad_create (datad *dsrc, datad *e, displayd *dsp, ggvisd *ggv, ggobid *gg)
   display_tailpipe (dspnew, FULL, gg);
 
   ggv->dpos = dnew;
+  clusters_set (ggv->dpos, gg);
 
   g_free(values);
   g_free(colnames);
@@ -216,22 +222,43 @@ ggv_pos_reinit (ggvisd *ggv)
   ggv_center_scale_pos_all (ggv);
 }
 
-/*
- * Definition of D
-*/
-void ggv_dsource_cb (GtkWidget *w, gpointer cbd)
+void ggv_task_cb (GtkToggleButton *button, PluginInstance *inst)
 {
-  PluginInstance *inst = (PluginInstance *)
-     gtk_object_get_data (GTK_OBJECT (w), "PluginInst");
-  ggvisd *ggv = ggvisFromInst (inst);
+  GtkWidget *w, *window;
 
-  ggv->Dtarget_source = (MDSDtargetSource) GPOINTER_TO_INT (cbd);
+  ggvisd *ggv = ggvisFromInst (inst); 
+  window = (GtkWidget *) inst->data;
+
+  if (button->active) {
+    if (strcmp (gtk_widget_get_name (GTK_WIDGET(button)), "MDS") == 0) {
+      ggv->mds_task = DissimAnalysis;
+    } else {
+      ggv->mds_task = GraphLayout;
+    }
+
+    w = widget_find_by_name (window, "MDS_WEIGHTS");
+    gtk_widget_set_sensitive (w, ggv->mds_task == GraphLayout);
+    w = widget_find_by_name (window, "MDS_COMPLETE");
+    gtk_widget_set_sensitive (w, ggv->mds_task == GraphLayout);
+
+    if (ggv->mds_task == DissimAnalysis)
+      gtk_clist_select_row (ggv->clist_dist, 0, 0);
+  }
 }
+
 void ggv_complete_distances_cb (GtkToggleButton *button, PluginInstance *inst)
 {
   ggvisd *ggv = ggvisFromInst (inst);
   ggv->complete_Dtarget = button->active;
 }
+void ggv_edge_weights_cb (GtkToggleButton *button, PluginInstance *inst)
+{
+  ggvisd *ggv = ggvisFromInst (inst);
+
+  ggv->Dtarget_source = (MDSDtargetSource) button->active;
+  if (ggv->Dtarget_source == VarValues)
+    gtk_clist_select_row (ggv->clist_dist, 0, 0);
+}
 
 /*
  * Required so that the histogram is properly drawn before mds_once
@@ -256,100 +283,6 @@ trans_dist_init_defaults (ggvisd *ggv)
   }
 }
 
-void ggv_compute_Dtarget_cb (GtkWidget *button, PluginInstance *inst)
-{
-  ggvisd *ggv = ggvisFromInst (inst);
-  GtkWidget *clist;
-  gint selected_var = -1;
-  datad *dsrc;
-  GtkWidget *window, *entry;
-  gchar *lbl;
-
-  /* This was set in the first clist in the preceding tab */
-  if (ggv->dsrc == NULL || ggv->dsrc->rowIds == NULL) {
-    g_printerr ("node set not correctly specified\n");
-    return;
-  }
-  if (ggv->e == NULL || ggv->e->edge.n == 0) {
-    g_printerr ("edge set not correctly specified\n");
-    return;
-  }
-
-  dsrc = ggv->dsrc; 
- 
-  /*-- allocate Dtarget --*/
-  arrayd_alloc (&ggv->Dtarget, dsrc->nrows, dsrc->nrows);
-
-  if (ggv->Dtarget_source == VarValues) {
-    clist = get_clist_from_object (GTK_OBJECT (button));
-    if (!clist) {
-      quick_message ("I can't identify a set of edges", false);
-      return;
-    }
-    /*-- this is the edgeset for distance computations, not
-         necessarily the edgeset to be displayed (eg, morsecode) --*/
-/* can override the setting of the second clist in the preceding tab */
-    ggv->e = gtk_object_get_data (GTK_OBJECT(clist), "datad");
-g_printerr ("e name = %s\n", ggv->e->name);
-    if (ggv->e == NULL) {
-g_printerr ("e is null\n");
-      quick_message ("I can't identify a set of edges", false);
-      return;
-    }
-    selected_var = get_one_selection_from_clist (clist, ggv->e);
-    if (selected_var == -1) {
-      quick_message ("Please specify a variable", false);
-      return;
-    }
-  }
-
-/*
-  if (ggv->e == NULL) {
-    if (!edgeset_add (dsp)) {
-      quick_message ("Please specify an edge set", false);
-      return;
-    } else {
-      ggv->e = dsp->e;
-    }
-  }
-*/
-
-  if (dsrc->rowIds == NULL) {
-    g_printerr ("Make sure the current display is a plot of the nodes.\n");
-    g_printerr ("  Currently nodes: %s edges: %s\n", dsrc->name, ggv->e->name);
-    return;
-  }
-
-  /*-- initalize Dtarget --*/
-  ggv_init_Dtarget (selected_var, ggv);
-/*
-  infinity = (gdouble) (2 * dsrc->nrows);
-  for (i=0; i<dsrc->nrows; i++) {
-    for (j=0; j<dsrc->nrows; j++)
-      ggv->Dtarget.vals[i][j] = infinity;
-    ggv->Dtarget.vals[i][i] = 0.0;
-  }
-*/
-
-  ggv_compute_Dtarget (selected_var, ggv);
-
-
-  /*-- update the entry to let people know Dtarget has been computed --*/
-  window = (GtkWidget *) inst->data;
-  entry = (GtkWidget *) gtk_object_get_data (GTK_OBJECT(window),
-    "DTARGET_ENTRY");
-  lbl = g_strdup_printf ("%d x %d", ggv->Dtarget.nrows, ggv->Dtarget.ncols);
-  gtk_entry_set_text (GTK_ENTRY (entry), lbl);
-  g_free (lbl);
-
-  trans_dist_init_defaults (ggv);
-
-  /*-- open display --*/
-  mds_open_display (inst);
-}
-
-/*-- --*/
-
 void
 mds_open_display (PluginInstance *inst)
 {
@@ -361,11 +294,12 @@ mds_open_display (PluginInstance *inst)
     quick_message ("I can't identify a distance matrix", false);
     return;
   }
+
   if (!ggv->dpos) {
     gint j;
     vartabled *vt;
     /*-- initialize, allocate and populate dpos --*/
-    ggv_datad_create (ggv->dsrc, ggv->e, gg->current_display, ggv, gg);
+    ggv_datad_create (ggv->dsrc, ggv->e, gg->current_display, inst);
     ggv_pos_init (ggv);
     /*-- update limits here? could force them to be -1, 1  --*/
     /*limits_set (true, true, ggv->dpos, gg);*/
@@ -385,8 +319,6 @@ mds_open_display (PluginInstance *inst)
   }
   
   window = (GtkWidget *) inst->data;
-  w = widget_find_by_name (window, "RunMDS");
-  gtk_widget_set_sensitive (w, true);
   w = widget_find_by_name (window, "Step");
   gtk_widget_set_sensitive (w, true);
 }
@@ -400,10 +332,99 @@ void mds_run_cb (GtkToggleButton *btn, PluginInstance *inst)
 {
   ggvisd *ggv = ggvisFromInst (inst);
   gboolean state = btn->active;
+  gint selected_var = -1;
+  datad *dsrc;
+  ggobid *gg = inst->gg;
 
-  if (ggv->Dtarget.nrows == 0) {
-    quick_message ("I can't identify a distance matrix", false);
-    return;
+  if (state) {
+
+    gboolean first_time = false;
+    gboolean new_dsrc = false;  /* node set */
+    gboolean new_weights = false;
+
+
+    /*
+     * make sure we have a node set, an edge set, and (if needed)
+     * a source of distances or edge weights 
+    */
+
+    /* Make sure the node set is present and can support edges */
+    if (ggv->dsrc == NULL || ggv->dsrc->rowIds == NULL) {
+      g_printerr ("node set not correctly specified\n");
+      return;
+    }
+    dsrc = ggv->dsrc;
+    /* Make sure there's an edge set */
+    if (ggv->clist_dist != (GtkCList *) NULL)
+      ggv->e = gtk_object_get_data (GTK_OBJECT(ggv->clist_dist), "datad");
+    if (ggv->e == NULL || ggv->e->edge.n == 0) {
+      g_printerr ("edge set not correctly specified\n");
+      return;
+    }
+    /*
+     * If we are using a distance vector or a vector of edge weights, make
+     * sure a variable has been selected.  (Do the variable values have
+     * to be non-negative?)
+    */
+    if (ggv->mds_task == DissimAnalysis || ggv->Dtarget_source == VarValues) {
+      selected_var = get_one_selection_from_clist (GTK_WIDGET(ggv->clist_dist),
+        ggv->e);
+      if (selected_var == -1) {
+        quick_message ("Please specify a variable", false);
+        return;
+      }
+    }
+
+    /* 
+     * Check the settings on a few flags so we'll know what has to
+     * be initialized or re-initialized
+     */
+    if (ggv->Dtarget.nrows == 0)
+      first_time = true;
+    else if (ggv->Dtarget.nrows != ggv->dsrc->nrows)
+      new_dsrc = true;
+    /* possible bug:  if the user resets this variable while mds is
+       running, the change won't show up here */
+    if (ggv->mds_task == DissimAnalysis || ggv->Dtarget_source == VarValues) {
+      if (selected_var != ggv->weight_var) {
+        new_weights = true;
+        ggv->weight_var = selected_var;
+      }
+    }
+
+    /*
+     * If this is the first time through or the node set changed,
+     * allocate space for the distance matrix. (There's no need
+     * to free it before allocating; arrayd_alloc handles that.)
+     */
+    if (first_time || new_dsrc)
+      arrayd_alloc (&ggv->Dtarget, dsrc->nrows, dsrc->nrows);
+
+    /*
+     * If anything changed, construct a distance matrix, and
+     * make sure we were successful.  Initialize (or reinitialize)
+     * some variables.  [We'll need more reinitialization here
+     * if new_weights or new_dsrc -- see what's initialized in
+     * init.c]
+    */
+    if (new_weights || first_time || new_dsrc) {
+      ggv_init_Dtarget (ggv->weight_var, ggv);  /* populate with INF */
+      ggv_compute_Dtarget (ggv->weight_var, ggv);
+      if (ggv->Dtarget.nrows == 0) {
+        quick_message ("I can't identify a distance matrix", false);
+        return;
+      }
+      g_printerr ("%d x %d\n", ggv->Dtarget.nrows, ggv->Dtarget.ncols);
+
+      trans_dist_init_defaults (ggv);
+    }
+
+    if (first_time) {
+      /*-- open display --*/
+      mds_open_display (inst);
+    }
+
+    ggv_Dtarget_histogram_update (ggv, gg);
   }
 
   mds_func (state, inst);
@@ -439,7 +460,7 @@ void mds_reinit_cb (PluginInstance *inst, guint action, GtkWidget *w)
   ggv_pos_reinit (ggv);
   update_ggobi (ggv, gg);
 }
-/*void mds_scramble_cb (GtkWidget *btn, PluginInstance *inst)*/
+
 void mds_scramble_cb (PluginInstance *inst, guint action, GtkWidget *w)
 {
   ggvisd *ggv = ggvisFromInst (inst);
@@ -552,7 +573,6 @@ void ggv_stepsize_cb (GtkAdjustment *adj, PluginInstance *inst)
 {
   ggvisd *ggv = ggvisFromInst (inst);
   ggv->stepsize = adj->value;
- g_printerr ("stepsize = %f\n", ggv->stepsize);
 }
 
 void ggv_dims_cb (GtkAdjustment *adj, PluginInstance *inst)
@@ -570,8 +590,7 @@ void ggv_dims_cb (GtkAdjustment *adj, PluginInstance *inst)
 
     if (ggv->running_p) mds_func (false, inst);
   
-    if (dim > ggv->dim) {  /*-- add variables as needed --*/
-
+    if (dim > ggv->maxdim) {  /*-- add variables as needed --*/
       arrayd_add_cols (&ggv->pos, dim);
       vectord_realloc (&ggv->pos_mean, dim);
 
@@ -586,12 +605,11 @@ void ggv_dims_cb (GtkAdjustment *adj, PluginInstance *inst)
         g_free (vname);
       }
       g_free (dtmp);
-
+      ggv->maxdim = MAX(ggv->dim, dim);
     }
+    ggv->dim = dim;
   }
 
-  ggv->dim = dim;
-
   if (ggv->dpos) {
     if (running) mds_func (true, inst);
   }
@@ -696,23 +714,48 @@ void ggv_kruskal_cb (GtkWidget *w, gpointer cbd)
   ggvisd *ggv = ggvisFromInst (inst);
 
   ggv->KruskalShepard_classic = (MDSKSInd) GPOINTER_TO_INT (cbd);
- g_printerr ("KruskalShepardInd = %d\n", ggv->KruskalShepard_classic);
 }
 
 
-void ggv_brush_groupsp_cb (GtkToggleButton *w, PluginInstance *inst)
+void ggv_groups_cb (GtkToggleButton *button,  gpointer cbd)
 {
-  ggvisd *ggv = ggvisFromInst (inst);
-  ggv->group_p = w->active;
-}
-void ggv_brush_groups_opt_cb (GtkWidget *w, gpointer cbd)
-{
-  PluginInstance *inst = (PluginInstance *)
-    gtk_object_get_data (GTK_OBJECT(w), "PluginInst");
-  ggvisd *ggv = ggvisFromInst (inst);
+  /* all_distances, within, between */
+  if (button->active) {
+    PluginInstance *inst = (PluginInstance *)
+      gtk_object_get_data (GTK_OBJECT(button), "PluginInst");
+    ggvisd *ggv = ggvisFromInst (inst);
 
-  /* within, between, anchorscales, anchorfixed */
-  ggv->group_ind = GPOINTER_TO_INT (cbd);
+    ggv->group_ind = GPOINTER_TO_INT (cbd);
+
+    /* If we just turned groups on, make sure anchor is off */
+    if (ggv->group_ind != 0) {
+      if (ggv->anchor_ind != 0) {
+        GtkWidget *window = (GtkWidget *) inst->data;
+        GtkWidget *w = widget_find_by_name (window, "ANCHOR_OFF");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), true);
+      }
+    }
+  }
+}
+void ggv_anchor_cb (GtkToggleButton *button, gpointer cbd)
+{
+  /* no_anchor, scaled, fixed */
+  if (button->active) {
+    PluginInstance *inst = (PluginInstance *)
+      gtk_object_get_data (GTK_OBJECT(button), "PluginInst");
+    ggvisd *ggv = ggvisFromInst (inst);
+
+    ggv->anchor_ind = GPOINTER_TO_INT (cbd);
+
+    /* If we just turned anchor on, make sure groups is off */
+    if (ggv->anchor_ind != 0) {
+      if (ggv->group_ind != 0) {
+        GtkWidget *window = (GtkWidget *) inst->data;
+        GtkWidget *w = widget_find_by_name (window, "GROUPS_OFF");
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(w), true);
+      }
+    }
+  }
 }
 
 void
