@@ -92,11 +92,11 @@ brush_once (gboolean force, splotd *sp, ggobid *gg)
 /*
  * Now paint.
 */
-  if (cpanel->br_point_targets) {
+  if (cpanel->br.point_targets) {
     changed = active_paint_points (sp, d, gg);
   }
 
-  if (cpanel->br_edge_targets) {
+  if (cpanel->br.edge_targets) {
     if (e != NULL)
       changed = active_paint_edges (sp, e, gg);
   }
@@ -156,8 +156,8 @@ reinit_transient_brushing (displayd *dsp, ggobid *gg)
   datad *d = dsp->d;
   datad *e = dsp->e;
   cpaneld *cpanel = &dsp->cpanel;
-  gboolean point_painting_p = (cpanel->br_point_targets != br_off);
-  gboolean edge_painting_p = (cpanel->br_edge_targets != br_off);
+  gboolean point_painting_p = (cpanel->br.point_targets != br_off);
+  gboolean edge_painting_p = (cpanel->br.edge_targets != br_off);
 
   g_assert (d->color.nels == d->nrows);
 
@@ -215,7 +215,7 @@ binning_permitted (displayd *display, ggobid *gg)
      /* If there is a function to determine this, call it. Otherwise just 
         get the value of binning_ok in the class. */
     GtkGGobiExtendedDisplayClass *klass;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(display)));
     if(klass->binningPermitted)
        return(klass->binningPermitted(display));
     return(klass->binning_ok);
@@ -235,14 +235,14 @@ binning_permitted (displayd *display, ggobid *gg)
   return permitted;
 }
 
-void
+gboolean
 brush_once_and_redraw (gboolean binningp, splotd *sp, displayd *display,
   ggobid *gg) 
 {
   cpaneld *cpanel = &display->cpanel;
   gboolean changed = false;
 
-  if (cpanel->brush_on_p) {
+  if (cpanel->br.brush_on_p) {
     changed = brush_once (!binningp, sp, gg);
 
     if (binningp && binning_permitted (display, gg)) {
@@ -265,9 +265,11 @@ brush_once_and_redraw (gboolean binningp, splotd *sp, displayd *display,
   } else {  /*-- we're not brushing, and we just need to redraw the brush --*/
     splot_redraw (sp, QUICK, gg);
   }
+
+  return(changed);
 }
 
-void
+gboolean
 brush_motion (icoords *mouse, gboolean button1_p, gboolean button2_p,
   cpaneld *cpanel, splotd *sp, ggobid *gg)
 {
@@ -301,7 +303,7 @@ brush_motion (icoords *mouse, gboolean button1_p, gboolean button2_p,
     brush_pos->y2 = mouse->y ;
   }
 
-  brush_once_and_redraw (true, sp, display, gg);  /* binning permitted */
+  return(brush_once_and_redraw (true, sp, display, gg));  /* binning permitted */
 }
 
 
@@ -336,7 +338,7 @@ brush_boundaries_set (cpaneld *cpanel,
   icoords *bin0 = &d->brush.bin0;
   icoords *bin1 = &d->brush.bin1;
 
-  if (cpanel->br_mode == BR_TRANSIENT) {
+  if (cpanel->br.mode == BR_TRANSIENT) {
     imin->x = MIN (bin0->x, obin0->x);
     imin->y = MIN (bin0->y, obin0->y);
     imax->x = MAX (bin1->x, obin1->x);
@@ -387,8 +389,8 @@ brush_draw_brush (splotd *sp, GdkDrawable *drawable, datad *d, ggobid *gg) {
 */
   displayd *display = sp->displayptr;
   cpaneld *cpanel = &display->cpanel;
-  gboolean point_painting_p = (cpanel->br_point_targets != br_off);
-  gboolean edge_painting_p = (cpanel->br_edge_targets != br_off);
+  gboolean point_painting_p = (cpanel->br.point_targets != br_off);
+  gboolean edge_painting_p = (cpanel->br.edge_targets != br_off);
   colorschemed *scheme = gg->activeColorScheme;
 
   brush_coords *brush_pos = &sp->brush_pos;
@@ -420,7 +422,7 @@ brush_draw_brush (splotd *sp, GdkDrawable *drawable, datad *d, ggobid *gg) {
     /*
      * highlight brush: but only in the current display
     */
-    if (cpanel->brush_on_p && display == gg->current_display) {
+    if (cpanel->br.brush_on_p && display == gg->current_display) {
       gdk_draw_rectangle (drawable, gg->plot_GC, false,
         x1-1, y1-1, (x2>x1)?(x2-x1+2):(x1-x2+2), (y2>y1)?(y2-y1+2):(y1-y2+2)); 
 
@@ -436,7 +438,7 @@ brush_draw_brush (splotd *sp, GdkDrawable *drawable, datad *d, ggobid *gg) {
     gdk_draw_line (drawable, gg->plot_GC,
       x1, y1 + (y2 - y1)/2, x2, y1 + (y2 - y1)/2 );
 
-    if (cpanel->brush_on_p) {
+    if (cpanel->br.brush_on_p) {
       gdk_draw_line (drawable, gg->plot_GC,
         x1 + (x2 - x1)/2 + 1, y1, x1 + (x2 - x1)/2 + 1, y2 );
       gdk_draw_line (drawable, gg->plot_GC,
@@ -476,7 +478,7 @@ update_glyph_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
 
   if (doit) {
     if (hit_by_brush[i]) {
-      switch (cpanel->br_mode) {
+      switch (cpanel->br.mode) {
 
         case BR_PERSISTENT:
           d->glyph.els[i].size = d->glyph_now.els[i].size = gg->glyph_id.size;
@@ -512,7 +514,7 @@ update_color_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
   if (!changed) {
     if (hit_by_brush[i])
       /*-- if persistent, compare against color instead of color_now --*/
-      doit = (cpanel->br_mode == BR_TRANSIENT) ?
+      doit = (cpanel->br.mode == BR_TRANSIENT) ?
                (d->color_now.els[i] != gg->color_id) :
                (d->color.els[i] != gg->color_id);
     else
@@ -525,7 +527,7 @@ update_color_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
   */
   if (doit) {
     if (hit_by_brush[i]) {
-      switch (cpanel->br_mode) {
+      switch (cpanel->br.mode) {
         case BR_PERSISTENT:
           d->color.els[i] = d->color_now.els[i] = gg->color_id;
         break;
@@ -568,7 +570,7 @@ update_hidden_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
 
   if (doit) {
     if (hit_by_brush[i]) {
-      switch (cpanel->br_mode) {
+      switch (cpanel->br.mode) {
         case BR_PERSISTENT:
           d->hidden.els[i] = d->hidden_now.els[i] = true;
         break;
@@ -613,7 +615,7 @@ update_selected_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
 
   if (doit) {
     if (hit_by_brush[i]) {
-      switch (cpanel->br_mode) {
+      switch (cpanel->br.mode) {
         case BR_PERSISTENT:
           d->hidden.els[i] = d->hidden_now.els[i] = false;
         break;
@@ -622,7 +624,7 @@ update_selected_vectors (gint i, gboolean changed, gboolean *hit_by_brush,
         break;
       }
     } else {
-      switch (cpanel->br_mode) {
+      switch (cpanel->br.mode) {
         case BR_PERSISTENT:
           d->hidden_now.els[i] = d->hidden.els[i];
         break;
@@ -662,7 +664,7 @@ build_symbol_vectors (cpaneld *cpanel, datad *d, ggobid *gg)
   displayd *display = (displayd *) sp->displayptr;
 
   if (GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
-    f = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass)->build_symbol_vectors; 
+    f = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(display)))->build_symbol_vectors; 
     if(f)
       changed = f(cpanel, d, gg);
   }
@@ -681,7 +683,7 @@ build_symbol_vectors (cpaneld *cpanel, datad *d, ggobid *gg)
           */
           j = d->rows_in_plot.els[ k = d->brush.binarray[ih][iv].els[m] ] ;
 
-          switch (cpanel->br_point_targets) {
+          switch (cpanel->br.point_targets) {
             case br_candg:  /*-- color and glyph --*/
               changed = update_color_vectors (j, changed,
                 d->pts_under_brush.els, d, gg);
@@ -700,10 +702,12 @@ build_symbol_vectors (cpaneld *cpanel, datad *d, ggobid *gg)
               changed = update_hidden_vectors (j, changed,
                 d->pts_under_brush.els, d, gg);
             break;
+	    /*
             case br_select:
               changed = update_selected_vectors (j, changed,
                 d->pts_under_brush.els, d, gg);
             break;
+	    */
             case br_off:
               ;
             break;
@@ -744,7 +748,7 @@ active_paint_points (splotd *sp, datad *d, ggobid *gg)
   g_assert (d->pts_under_brush.nels == d->nrows);
 
   if (GTK_IS_GGOBI_EXTENDED_SPLOT(sp)) {
-    f = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT(sp)->klass)->active_paint_points;
+    f = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(GTK_OBJECT(sp)))->active_paint_points;
     if(f)
        d->npts_under_brush = f(sp, d, gg);
   }
@@ -773,8 +777,8 @@ active_paint_points (splotd *sp, datad *d, ggobid *gg)
            * cases; otherwise it's ok (I think).
           */
           if (d->hidden_now.els[pt] &&
-            (cpanel->br_point_targets != br_hide &&
-            cpanel->br_point_targets != br_select))
+            (cpanel->br.point_targets != br_hide 
+	     /* && cpanel->br.point_targets != br_select */))
           {
               continue;
           }
@@ -791,7 +795,7 @@ active_paint_points (splotd *sp, datad *d, ggobid *gg)
 
   changed = false;
 
-  if (cpanel->brush_on_p) {
+  if (cpanel->br.brush_on_p) {
     if (gg->linkby_cv) {
       /*-- link by categorical variable --*/
       changed = build_symbol_vectors_by_var (cpanel, d, gg);
@@ -872,7 +876,7 @@ build_edge_symbol_vectors (cpaneld *cpanel, datad *e, ggobid *gg)
 */
   for (i=0; i < e->edge.n ; i++) {
 
-    switch (cpanel->br_edge_targets) {
+    switch (cpanel->br.edge_targets) {
       case br_candg:  /*-- color and glyph --*/
         changed = update_color_vectors (i, changed,
           e->edge.xed_by_brush.els, e, gg);
@@ -891,10 +895,12 @@ build_edge_symbol_vectors (cpaneld *cpanel, datad *e, ggobid *gg)
         changed = update_hidden_vectors (i, changed,
           e->edge.xed_by_brush.els, e, gg);
       break;
+      /* disabled
       case br_select:
         changed = update_selected_vectors (i, changed,
           e->edge.xed_by_brush.els, e, gg);
       break;
+      */
       case br_off:
         ;
       break;
@@ -931,7 +937,7 @@ active_paint_edges (splotd *sp, datad *e, ggobid *gg)
   }
 
   changed = false;
-  if (cpanel->brush_on_p) {
+  if (cpanel->br.brush_on_p) {
     if (gg->linkby_cv) {
       /*-- link by categorical variable --*/
       /*changed = build_symbol_vectors_by_var (cpanel, e, gg);*/
