@@ -285,6 +285,9 @@ redraw_line_display (GtkWidget *w, ggobid *gg)
   margin = gg->color_ui.margin;
   spacing = gg->color_ui.spacing;
 
+  if (gg->plot_GC == NULL)
+    init_plot_GC (w->window, gg);
+
   gdk_gc_set_foreground (gg->plot_GC, &scheme->rgb_bg);
   gdk_draw_rectangle (w->window, gg->plot_GC,
     true, 0, 0, w->allocation.width, w->allocation.height);
@@ -399,7 +402,7 @@ set_color_id (GtkWidget *w, GdkEventButton *event, ggobid *gg)
     set_color_fg (w, event, gg);
 
   splot_redraw (gg->current_splot, 
-                GTK_GGOBI_SPLOT_CLASS(GTK_OBJECT(gg->current_splot)->klass)->redraw,
+                GTK_GGOBI_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(gg->current_splot))->redraw,
                 gg);  /*-- redraw brush --*/
 
   return FALSE;
@@ -565,22 +568,42 @@ color_changed_cb (GtkWidget *colorsel, ggobid *gg)
 
   /* Allocate color */
   if (gdk_color_alloc (cmap, &gdk_color)) {
-
     if (gg->color_ui.current_da == gg->color_ui.bg_da) {
-      scheme->rgb_bg = gdk_color;
+
+      scheme->rgb_bg.pixel = gdk_color.pixel;
+      scheme->rgb_bg.red = gdk_color.red;
+      scheme->rgb_bg.green = gdk_color.green;
+      scheme->rgb_bg.blue = gdk_color.blue;
+
       redraw_bg (gg->color_ui.bg_da, gg);
     } else if (gg->color_ui.current_da == gg->color_ui.accent_da) {
-      scheme->rgb_accent = gdk_color;
+
+      scheme->rgb_accent.pixel = gdk_color.pixel;
+      scheme->rgb_accent.red = gdk_color.red;
+      scheme->rgb_accent.green = gdk_color.green;
+      scheme->rgb_accent.blue = gdk_color.blue;
+
       redraw_accent (gg->color_ui.accent_da, gg);
     } else if (gg->color_ui.current_da == gg->color_ui.hidden_da) {
-      scheme->rgb_hidden = gdk_color;
+
+      scheme->rgb_hidden.pixel = gdk_color.pixel;
+      scheme->rgb_hidden.red = gdk_color.red;
+      scheme->rgb_hidden.green = gdk_color.green;
+      scheme->rgb_hidden.blue = gdk_color.blue;
+
       redraw_hidden (gg->color_ui.hidden_da, gg);
     } else {
-      gg->activeColorScheme->rgb[gg->color_id] = gdk_color;
+
+      gg->activeColorScheme->rgb[gg->color_id].pixel = gdk_color.pixel;
+      gg->activeColorScheme->rgb[gg->color_id].red = gdk_color.red;
+      gg->activeColorScheme->rgb[gg->color_id].green = gdk_color.green;
+      gg->activeColorScheme->rgb[gg->color_id].blue = gdk_color.blue;
+
       redraw_fg (gg->color_ui.fg_da[gg->color_id], gg->color_id, gg);
     }
 
     redraw_symbol_display (gg->color_ui.symbol_display, gg);
+    redraw_line_display (gg->color_ui.line_display, gg);
 
     if (sp->da != NULL) {
       gboolean rval = false;
@@ -753,6 +776,14 @@ symbol_window_redraw (ggobid *gg)
   }
 }
 
+void
+close_symbol_window_cb(GtkWidget *w, GdkEventButton *event, ggobid *gg)
+{
+	fprintf(stderr, "Closing the color scheme window\n");fflush(stderr);
+   gtk_widget_destroy(gg->color_ui.symbol_window);
+   gg->color_ui.symbol_window = NULL;
+}
+
 /*------------------------------------------------------------------------*/
 /*                    build the window                                    */
 /*------------------------------------------------------------------------*/
@@ -787,6 +818,13 @@ make_symbol_window (ggobid *gg) {
                         GTK_SIGNAL_FUNC (delete_symbol_window_cb),
                         (gpointer) gg);
 
+
+    /* Track when the ggobid instance is closed and shut this one down too. */
+    gtk_signal_connect (GTK_OBJECT (gg->main_window),
+                        "delete_event",
+                        GTK_SIGNAL_FUNC (close_symbol_window_cb),
+                        (gpointer) gg);
+
     vbox = gtk_vbox_new (false, 2);
     gtk_container_add (GTK_CONTAINER (gg->color_ui.symbol_window), vbox);
 
@@ -937,7 +975,7 @@ make_symbol_window (ggobid *gg) {
       PSIZE, PSIZE);
     gtk_tooltips_set_tip (GTK_TOOLTIPS (gg->tips),
       gg->color_ui.bg_da,
-      "Double click to reset background color (Note: your color selection will have no visible effect unless you also reset the 'Value')",
+      "Double click to reset background color (Note: your color selection will have no visible effect unless the 'Value' is >>0; look to the right of the color wheel.)",
       NULL);
     gtk_widget_set_events (gg->color_ui.bg_da,
                            GDK_EXPOSURE_MASK
