@@ -39,6 +39,10 @@ DisplayOptions DefaultDisplayOptions = {
 */
                                        };
 
+
+void set_display_option(gboolean active, guint action, displayd *display, ggobid *gg);
+
+
 /*-- debugging utility --*/
 /*
 static void
@@ -100,7 +104,14 @@ display_plot_allbutone (displayd *display, splotd *splot,
 static void
 display_edges_directed_show (displayd *display, gboolean show)
 {
-  GtkWidget *w = widget_find_by_name (display->edge_menu,
+  GtkWidget *w;
+
+  if(!display->edge_menu) {
+     /* set_display_options doesn't know whether to call this or not. */
+    return;
+  }
+
+  w  = widget_find_by_name (display->edge_menu,
     "DISPLAYMENU:edges_d");
 
   if (!show && display->options.edges_directed_show_p && w) {
@@ -149,6 +160,21 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
   ggobid *gg = GGobiFromWidget(GTK_WIDGET(w), true);
   displayd *display = (displayd *)
     gtk_object_get_data (GTK_OBJECT (w), "display");
+
+  set_display_option(w->active, action, display, gg);
+}
+
+
+/*
+
+We need to allow people to programmatically change a setting and force the update. 
+The current framework is all based on GUI events and so we don't update the GUI components 
+here as we assume they are set appropriately.  A Model View Controller approach is needed.
+
+*/
+void
+set_display_option(gboolean active, guint action, displayd *display, ggobid *gg)
+{
   gchar *title;
   gint ne = 0;
   datad *onlye = NULL;
@@ -174,7 +200,7 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
 
   switch (action) {
     case DOPT_POINTS:
-      display->options.points_show_p = w->active;
+      display->options.points_show_p = active;
       display_plot (display, FULL, gg);
     break;
 
@@ -187,8 +213,8 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
    have been correctly set, the problem vanishes!  -- dfs
 */
 
-      display->options.edges_undirected_show_p = w->active;
-      if (w->active) {
+      display->options.edges_undirected_show_p = active;
+      if (active) {
         display_edges_directed_show (display, false);
         display_edges_arrowheads_show (display, false);
       }
@@ -207,8 +233,8 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
     break;
     case DOPT_EDGES_D:  /*-- directed: both edges and arrowheads --*/
 
-      display->options.edges_directed_show_p = w->active;
-      if (w->active) {
+      display->options.edges_directed_show_p = active;
+      if (active) {
         display_edges_undirected_show (display, false);
         display_edges_arrowheads_show (display, false);
       }
@@ -226,8 +252,8 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
       display_plot (display, FULL, gg);
     break;
     case DOPT_EDGES_A:  /*-- arrowheads only --*/
-      display->options.edges_arrowheads_show_p = w->active;
-      if (w->active) {
+      display->options.edges_arrowheads_show_p = active;
+      if (active) {
         display_edges_directed_show (display, false);
         display_edges_undirected_show (display, false);
       }
@@ -247,7 +273,7 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
     break;
 
     case DOPT_WHISKERS:
-      display->options.whiskers_show_p = w->active;
+      display->options.whiskers_show_p = active;
       display_plot (display, FULL, gg);
     break;
 /*
@@ -277,35 +303,35 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
 */
 
     case DOPT_AXES:
-      display->options.axes_show_p = w->active;
+      display->options.axes_show_p = active;
 
       if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
         GtkGGobiExtendedDisplayClass *klass;
-        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
         if(klass->set_show_axes_option)
-          klass->set_show_axes_option(display, w->active);
+          klass->set_show_axes_option(display, active);
       }
     break;
 
     case DOPT_AXESLAB:
-      display->options.axes_label_p = w->active;
+      display->options.axes_label_p = active;
 
       if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
         GtkGGobiExtendedDisplayClass *klass;
-        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
         if(klass->set_show_axes_label_option)
-          klass->set_show_axes_label_option(display, w->active);
+          klass->set_show_axes_label_option(display, active);
       }      
     break;
 
     case DOPT_AXESVALS:
-      display->options.axes_values_p = w->active;
+      display->options.axes_values_p = active;
 
       if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
         GtkGGobiExtendedDisplayClass *klass;
-        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
         if(klass->set_show_axes_values_option)
-          klass->set_show_axes_values_option(display, w->active);
+          klass->set_show_axes_values_option(display, active);
       }
     break;
 
@@ -325,6 +351,56 @@ display_options_cb (GtkCheckMenuItem *w, guint action)
   }
 }
 
+
+/**
+ This is a start at allowing programmatic specification
+of options. Here, we set them in place (i.e. in
+display->options) and then force them to be applied.
+This does not update the menus.
+*/
+void
+set_display_options(displayd *display, ggobid *gg)
+{
+    int i;
+    gboolean active;
+    DisplayOptions *options = &display->options;
+    for(i = DOPT_POINTS ; i <=  DOPT_WHISKERS; i++) {
+       if (i == DOPT_EDGES_U || i == DOPT_EDGES_D ||  i == DOPT_EDGES_A)
+         if(!display->edge_menu)
+   	   continue;
+
+         switch(i) {
+		 case DOPT_POINTS:
+			 active = options->points_show_p;
+			 break;
+		 case DOPT_AXES:
+			 active = options->axes_show_p;
+			 break;
+		 case DOPT_AXESLAB:
+			 active = options->axes_label_p;
+			 break;
+		 case DOPT_AXESVALS:
+			 active = options->axes_values_p;
+			 break;
+		 case DOPT_EDGES_U:
+			 active = options->edges_undirected_show_p;
+			 break;
+		 case DOPT_EDGES_A:
+			 active = options->edges_arrowheads_show_p;
+			 break;
+		 case DOPT_EDGES_D:
+			 active = options->edges_directed_show_p;
+			 break;
+		 case DOPT_WHISKERS:
+			 active = options->whiskers_show_p;
+			 break;
+	 }
+
+         set_display_option(active, i, display, gg);
+    }
+}
+
+
 void
 display_print_cb (displayd *display, guint action, GtkWidget *w) 
 {
@@ -389,6 +465,7 @@ display_alloc_init (gboolean missing_p, datad *d, ggobid *gg)
   displayd *display;
 
   display = gtk_type_new(GTK_TYPE_GGOBI_WINDOW_DISPLAY);
+  GTK_GGOBI_WINDOW_DISPLAY(display)->useWindow = true;
   display_set_values(display, d, gg);
 
   return (display);
@@ -409,7 +486,7 @@ display_add (displayd *display, ggobid *gg)
     return(-1);
   }
 
-  if (GTK_IS_GGOBI_WINDOW_DISPLAY(display)) {
+  if (GTK_IS_GGOBI_WINDOW_DISPLAY(display)  && GTK_GGOBI_WINDOW_DISPLAY(display)->useWindow) {
     GGobi_widget_set(GTK_GGOBI_WINDOW_DISPLAY(display)->window, gg, true);
     if(g_list_length(display->splots))
           display_set_current (display, gg);  /*-- this may initialize the mode --*/
@@ -614,7 +691,7 @@ display_set_current (displayd *new_display, ggobid *gg)
       GTK_IS_GGOBI_WINDOW_DISPLAY(gg->current_display))
   {
     title = computeTitle (false, gg->current_display, gg);
-    if (title) {
+    if (title && GTK_GGOBI_WINDOW_DISPLAY(gg->current_display)->window) {
       gtk_window_set_title (GTK_WINDOW (GTK_GGOBI_WINDOW_DISPLAY(gg->current_display)->window), title);
       g_free (title); 
     }
@@ -626,7 +703,7 @@ display_set_current (displayd *new_display, ggobid *gg)
      /* Allow the extended display to override the submenu_destroy call.
         If it doesn't provide a method, then call submenu_destroy. */
       void (*f)(displayd *dpy, GtkWidget *) =
-        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(gg->current_display)->klass)->display_unset;
+        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(gg->current_display))->display_unset;
       if(f)
         f(gg->current_display, gg->viewmode_item);
       else
@@ -636,17 +713,19 @@ display_set_current (displayd *new_display, ggobid *gg)
 
   /* Now do the setup for the new display.  */
   if (GTK_IS_GGOBI_WINDOW_DISPLAY(new_display)) {
-    title = computeTitle (true, new_display, gg);
-    if (title) {
-      gtk_window_set_title (GTK_WINDOW (GTK_GGOBI_WINDOW_DISPLAY(new_display)->window), title);   
-      g_free (title); 
+    if(GTK_GGOBI_WINDOW_DISPLAY(new_display)->useWindow) {
+      title = computeTitle (true, new_display, gg);
+      if (title) {
+        gtk_window_set_title (GTK_WINDOW (GTK_GGOBI_WINDOW_DISPLAY(new_display)->window), title);   
+        g_free (title); 
+      }
     }
 
     if(GTK_IS_GGOBI_EXTENDED_DISPLAY(new_display)) {
        /* Allow the extended display to override the submenu_destroy call.
           If it doesn't provide a method, then call submenu_destroy. */
       void (*f)(displayd *dpy, ggobid *gg) =
-        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(new_display)->klass)->display_set;
+        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(new_display))->display_set;
       if(f)
         f(new_display, gg);
     }
@@ -662,7 +741,7 @@ display_set_current (displayd *new_display, ggobid *gg)
    * the variable manipulation table.
   */
   varpanel_show_page (gg->current_display, gg);
-  vartable_show_page (gg->current_display, gg);
+  vartable_show_page (gg->current_display->d, gg);
 
   varpanel_tooltips_set (gg->current_display, gg);
 
@@ -680,7 +759,15 @@ computeTitle (gboolean current_p, displayd *display, ggobid *gg)
 {
   gint n;
   gchar *title = NULL, *description;
-  const char *tmp = NULL, *stars;
+  const char *tmp = NULL;
+
+  /*
+     I was going to use this value to avoid displaying "(current)" when
+     there's only one display, but the value of ndisplays is almost never
+     right.  This is weird.  dfs.
+  gint ndisplays = g_list_length(gg->displays);
+  g_printerr ("ndisplays %d\n", ndisplays);
+  */
 
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
     tmp = gtk_display_title_label(display);
@@ -695,11 +782,12 @@ computeTitle (gboolean current_p, displayd *display, ggobid *gg)
     description = GGOBI (getDescription)(gg);
   }
 
-  stars = current_p ? "***" : "";
-  n = strlen (tmp) + strlen (description) + 5 + (current_p ? strlen(stars)*2 : 0);
+  n = strlen (tmp) + strlen (description) + 5 +
+   (current_p ? strlen("(current)") : 0);
   title = (gchar *) g_malloc(sizeof(gchar) * n);
   memset (title, '\0', n);
-  sprintf (title, "%s: %s %s %s", description, stars, tmp, stars);
+  sprintf (title, "%s: %s %s", description, tmp,
+   (current_p ? "(current)":""));
   g_free (description);
 
   return (title);
@@ -755,7 +843,7 @@ display_tailpipe (displayd *display, RedrawStyle type, ggobid *gg)
 
     if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
       void (*f)(gboolean, displayd *, splotd *, ggobid *);
-      f = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass)->ruler_ranges_set;
+      f = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display))->ruler_ranges_set;
       if(f) {
         f(GTK_WIDGET_VISIBLE (display->hrule) ||
           GTK_WIDGET_VISIBLE (display->vrule),
@@ -801,7 +889,7 @@ gboolean
 isEmbeddedDisplay (displayd *dpy)
 {
   gboolean ans = false;
-  ans = (GTK_IS_GGOBI_WINDOW_DISPLAY(dpy) == false);
+  ans = (GTK_IS_GGOBI_WINDOW_DISPLAY(dpy) == false || GTK_GGOBI_WINDOW_DISPLAY(dpy)->useWindow);
 
   return (ans);
 }
@@ -818,7 +906,7 @@ display_type_handles_action (displayd *display, PipelineMode viewmode)
   PipelineMode v = viewmode;
 
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
-    handles = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass)->handles_action(display, v);
+    handles = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display))->handles_action(display, v);
   } 
 
   return handles;
