@@ -16,7 +16,7 @@
 #include "vars.h"
 #include "externs.h"
 
-static gint exclusion_notebook_adddata_cb(ggobid *, datad *, void* notebook);
+static void exclusion_notebook_adddata_cb(ggobid *, datad *, void* notebook);
 
 static void destroyit (gboolean kill, ggobid * gg)
 {
@@ -255,7 +255,7 @@ cluster_symbol_cb(GtkWidget * w, GdkEventExpose * event, gpointer cbd)
   gboolean rval = false;
   gint nclusters = symbol_table_populate(d);
   gboolean proceed = true;
-  gint targets = cpanel->br_point_targets;
+  gint targets = cpanel->br.point_targets;
   gint nd = g_slist_length(gg->d);
 
   g_assert (d->clusterid.nels == d->nrows);
@@ -357,7 +357,7 @@ void cluster_add(gint k, datad * d, ggobid * gg)
 
   d->clusvui[k].da = gtk_drawing_area_new();
 #if GTK_MAJOR_VERSION == 2
-  gtk_widget_set_double_buffered(d->clusvui[j].da, false);
+  gtk_widget_set_double_buffered(d->clusvui[k].da, false);
 #endif
   gtk_drawing_area_size(GTK_DRAWING_AREA(d->clusvui[k].da),
     dawidth, dawidth);
@@ -448,7 +448,8 @@ static void update_cb(GtkWidget * w, ggobid * gg)
   cluster_window_open(gg);
 }
 
-static gboolean nclusters_changed(ggobid * gg)
+static gboolean 
+nclusters_changed(ggobid * gg)
 {
   datad *d;
   gint k, nrows = 0;
@@ -479,9 +480,7 @@ static gboolean nclusters_changed(ggobid * gg)
 
 void cluster_table_update (datad * d, ggobid * gg)
 {
-  if (gg->cluster_ui.window == NULL) {
-    ;
-  } else {
+  if (gg->cluster_ui.window != NULL) {
     if (nclusters_changed(gg)) {   /*-- for any of the datad's --*/
       cluster_window_open(gg);
     } else {
@@ -490,11 +489,11 @@ void cluster_table_update (datad * d, ggobid * gg)
   }
 }
 
-static gint
+static void
 exclusion_notebook_adddata_cb (ggobid *gg, datad * d, void* notebook)
 {
   cluster_window_open (gg);
-  return true;  /* risky -- will this prevent other guys from getting it?  --*/
+  return;  /* Should this return a boolean? */
 }
 
 CHECK_EVENT_SIGNATURE(exclusion_notebook_adddata_cb, datad_added_f)
@@ -699,7 +698,7 @@ void cluster_window_open(ggobid * gg)
   btn = gtk_button_new_with_label("Close");
   gtk_signal_connect(GTK_OBJECT(btn), "clicked",
     GTK_SIGNAL_FUNC(close_btn_cb), (gpointer) gg);
-  gtk_box_pack_start(GTK_BOX(vbox), btn, true, true, 0);
+  gtk_box_pack_start(GTK_BOX(vbox), btn, false, false, 0);
 
   gtk_widget_show_all(gg->cluster_ui.window);
 
