@@ -41,11 +41,20 @@ void GGOBI(splot_release)(splotd *sp, displayd *display, ggobid *gg);
 void GGOBI(data_release)(datad *, ggobid *gg);
 void GGOBI(vartable_free)(datad *, ggobid *gg);
 void GGOBI(vardatum_free)(vartabled *var, ggobid *gg);
-
+void GGOBI(setErrorHandlers)(FatalErrorHandler err);
 #ifdef __cplusplus
 }
 #endif
 
+
+void
+GGOBI(setErrorHandlers)(FatalErrorHandler err)
+{
+    if(!err)
+       return;
+    FatalError = err;
+}
+
 const gchar *
 GGOBI(getFileName) (ggobid *gg)
 {
@@ -56,7 +65,7 @@ const gchar *
 GGOBI(setFileName) (const gchar *fileName, DataMode data_mode, ggobid *gg)
 {
   const gchar *old = g_strdup(GGOBI(getFileName)(gg));
-  fileset_read_init(fileName, gg);
+  fileset_read_init(fileName, NULL, NULL, gg);
   display_menu_build (gg);
   return(old);
 }
@@ -77,20 +86,50 @@ GGOBI(setDataMode) (DataMode newMode, ggobid *gg)
   return(old);
 }
 
+#if 0
+/*XXX */
 const gchar * const 
 GGOBI(getDataModeDescription)(DataMode mode)
 {
-  extern const gchar * const DataModeNames[];
   return(DataModeNames[mode]);
 }
+#endif
 
-const gchar *const *
+
+/**
+ Caller should now free the return value, but not the elements of the
+ array.
+ This is computed dynamically by querying each of the input plugins 
+ for its collection of mode names.
+*/
+const gchar **
 GGOBI(getDataModeNames)(int *n)
 {
-  extern const gchar * const DataModeNames[num_data_modes];
-  if(n)
-    *n = num_data_modes;
-  return(DataModeNames);
+   int ctr = 0, num, k, i;
+   GList *plugins;
+   const gchar **ans;
+   GGobiPluginInfo *plugin;
+
+   plugins = sessionOptions->info->inputPlugins;
+   num = g_list_length(plugins);
+   for(i = 0; i < num ; i++) {
+      plugin = g_list_nth_data(plugins, i);  
+      ctr += plugin->info.i->numModeNames;
+   }
+
+   ans = (const gchar **) g_malloc(sizeof(gchar *) * ctr);
+   ctr = 0;
+   for(i = 0; i < num ; i++) {
+      plugin = g_list_nth_data(plugins, i);  
+      for(k = 0; k < plugin->info.i->numModeNames; k++) {
+         ans[ctr++] = plugin->info.i->modeNames[k];
+      }
+   }
+
+   if(n)
+     *n = ctr;
+
+   return(ans);
 }
 
 
@@ -160,6 +199,26 @@ GGobi_setMissingValueIdentifier(MissingValue_p f)
 }
 
 
+static const gchar * const DefaultRowNames = NULL;
+
+const gchar ** const
+getDefaultRowNamesPtr()
+{
+  return((const gchar ** const) &DefaultRowNames);
+}
+
+void
+setRowNames(datad *d, gchar **rownames)
+{
+    int i;
+    gchar *lbl;
+    for (i = 0; i < d->nrows ; i++) {
+        lbl = (rownames != (gchar **) &DefaultRowNames && rownames != (gchar **) NULL && rownames[i] != (gchar *) NULL) ?
+          g_strdup (rownames[i]) : g_strdup_printf ("%d", i+1);
+        g_array_append_val (d->rowlab, lbl);
+    }
+}
+
 /*
   An initial attempt to allow new data to be introduced
   to the Ggobi session, replacing the existing contents.
@@ -230,7 +289,9 @@ GGOBI(setData)(gdouble *values, gchar **rownames, gchar **colnames,
   br_hidden_alloc (d);
   br_hidden_init(d);
 
-  for (j = 0; j < nc ; j++) {
+  if(values && d->vartable) {
+      /* the person who created the datad is taking care of populating it.*/
+   for (j = 0; j < nc ; j++) {
     vt = vartable_element_get (j, d);
     vt->collab = (colnames != NULL && colnames[j] != NULL) ?
       g_strdup(colnames[j]) : g_strdup_printf("V%d", j+1);
@@ -245,12 +306,18 @@ GGOBI(setData)(gdouble *values, gchar **rownames, gchar **colnames,
         /* g_free (lbl); */
       }
 
-      if(GGobiMissingValue && GGobiMissingValue(values[i + j*nr]))
+      if(values) {
+        if(GGobiMissingValue && GGobiMissingValue(values[i + j*nr]))
          setMissingValue(i, j, d, vt);
-      else
+        else
          d->raw.vals[i][j] = values[i + j*nr];
+      }
     }
-  }
+   }
+  } 
+
+  if(rownames && d->rowlab->len == 0) 
+    setRowNames(d, rownames);
 
 
   /* Now recompute and display the top plot. */
@@ -262,14 +329,12 @@ GGOBI(setData)(gdouble *values, gchar **rownames, gchar **colnames,
          This looks very dangerous. Should use g_list_remove();
        */
     gg->displays->next = NULL;
-
   }
   display_menu_build (gg);
 }
 
 
 /* These are all for freeing the currently held data. */
-
 void
 GGOBI(displays_release)(ggobid *gg)
 {
@@ -387,8 +452,8 @@ GGOBI(newScatmat) (gint *rows, gint *columns, gint nr, gint nc,
 {
   displayd *display; 
 
-  display = scatmat_new (false, nr, rows, nc, columns, d, gg);
-/*XX  display_add (display, gg); the caller should add this display. */
+  display = scatmat_new (NULL, false, nr, rows, nc, columns, d, gg);
+  display_add (display, gg); /*XX  the caller should add this display. */
 
   return (display);
 }
@@ -398,7 +463,7 @@ GGOBI(newParCoords)(gint *vars, gint numVars, datad *d, ggobid *gg)
 {
   displayd *display = NULL;
 
-  display = parcoords_new (false, numVars, vars, d, gg);
+  display = parcoords_new (display, false, numVars, vars, d, gg);
   display_add (display, gg);
 
   return (display);
@@ -409,7 +474,7 @@ GGOBI(newTimeSeries)(gint *yvars, gint numVars, datad *d, ggobid *gg)
 { 
   displayd *display = NULL; 
  
-  display = tsplot_new (false, numVars, yvars, d, gg); 
+  display = tsplot_new (display, false, numVars, yvars, d, gg); 
   display_add (display, gg); 
  
   return (display);
@@ -444,7 +509,7 @@ GGOBI(getViewTypeName)(displayd *dpy)
 /*
  or use gtk_type_name(GTK_OBJECT_TYPE(dpy))
  */
-  val = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(dpy)->klass)->treeLabel;
+  val = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(dpy))->treeLabel;
 
   return (val);
 }
@@ -963,6 +1028,12 @@ GGOBI(close)(ggobid *gg, gboolean closeWindow)
   if (gg->vartable_ui.window)
     gtk_widget_destroy (gg->vartable_ui.window);
 
+  if (gg->color_ui.symbol_window) 
+    gtk_widget_destroy (gg->color_ui.symbol_window);
+
+  if (gg->wvis.window) 
+    gtk_widget_destroy (gg->wvis.window);
+
   gg->close_pending = false;
   /* Now fix up the list of ggobi's */
   val = ggobi_remove(gg) != -1;
@@ -1144,42 +1215,42 @@ GGOBI(getColorName)(gint cid, ggobid *gg, gboolean inDefault)
 }
 
 
-/*-- this is really addRealVariable --*/
-gint
-GGOBI(addVariable)(gdouble *vals, gint num, gchar *name,
-  gboolean update, datad *d, ggobid *gg)
+static gint
+addVariableInternal(gdouble *vals, gint num, gchar *name,
+		    gchar **levels, gint *values, gint *counts, gint numLevels,
+		    gboolean update, datad *d, ggobid *gg)
 {
-  gint i;
-
   if (d->ncols < 1) {
-    gchar ** rnames = (gchar **)g_malloc(sizeof(gchar*) * num);
-    for (i = 0; i < num; i++) {
-      rnames[i] = (gchar *) g_malloc (sizeof (gchar)*7);
-      rnames[i] = g_strdup_printf ("%d", i+1);
-/*    sprintf(rnames[i],"%d",i+1);*/
-    }
+    gchar ** rnames = (gchar **) &DefaultRowNames; 
     /* May want the final false here to be true as it causes the 
        creation of a plot. Probably not, but just mention it here
        so we don't forget.
      */
-    GGOBI(setData)(vals, rnames, &name, num, 1, d, false, gg, NULL, false, d->input);
-    for (i = 0; i < num; i++)
-      g_free (rnames[i]);
-    g_free (rnames);
-  } else {
+    GGOBI(setData)(NULL, rnames, &name, num, d->ncols, d, false, gg, NULL, false, d->input); 
+    datad_init(d, gg, false);
+  }
+
     if (num > d->nrows) {
-      num = d->nrows;
+      num =  d->nrows;
       /* Add a warning here. */
     }
     newvar_add_with_values (vals, num, name,
-      real, 0, NULL, NULL, NULL,
-      d, gg);
-  }
+			    numLevels > 0 ? categorical : real, 
+			    numLevels, levels, values, counts,
+			    d, gg);
 
-  if (update)
+  if(update)
     gdk_flush();
 
-  return (d->ncols - 1);  /*-- incremented by clone_vars --*/
+  return(d->ncols-1);
+}
+
+/*-- this is really addRealVariable --*/
+gint
+GGOBI(addVariable)(gdouble *vals, gint num, gchar *name,
+		   gboolean update, datad *d, ggobid *gg)
+{
+  return(addVariableInternal(vals, num, name, NULL, NULL, NULL, 0, update, d, gg));
 }
 
 /*
@@ -1191,44 +1262,11 @@ only very limted use of the event we emit.    dfs  2/7
 */
 gint
 GGOBI(addCategoricalVariable)(gdouble *vals, gint num, gchar *name,
-  gchar **levels, gint *values, gint *counts, gint numLevels,
-  gboolean update, datad *d, ggobid *gg)
+			      gchar **levels, gint *values, gint *counts, gint numLevels,
+			      gboolean update, datad *d, ggobid *gg)
 {
-  gint i;
-/*
-  int ans;
-  ans = GGOBI(addVariable)(vals, num, name, false, d, gg);
-*/
 
-  if (d->ncols < 1) {
-    gchar ** rnames = (gchar **)g_malloc(sizeof(gchar*) * num);
-    for (i = 0; i < num; i++) {
-      rnames[i] = (gchar *) g_malloc (sizeof (gchar)*7);
-      rnames[i] = g_strdup_printf ("%d", i+1);
-/*    sprintf(rnames[i],"%d",i+1);*/
-    }
-    /* May want the final false here to be true as it causes the 
-       creation of a plot. Probably not, but just mention it here
-       so we don't forget.
-     */
-    GGOBI(setData)(vals, rnames, &name, num, 1, d, false, gg, NULL, false, d->input);
-    for (i = 0; i < num; i++)
-      g_free (rnames[i]);
-    g_free (rnames);
-  } else {
-    if (num > d->nrows) {
-      num =  d->nrows;
-      /* Add a warning here. */
-    }
-    newvar_add_with_values (vals, num, name,
-      categorical, numLevels, levels, values, counts,
-      d, gg);
-  }
-
-  if(update)
-    gdk_flush();
-
-  return(d->ncols-1);
+  return(addVariableInternal(vals, num, name, levels, values, counts, numLevels, update, d, gg));
 }
 
 
@@ -1284,6 +1322,7 @@ GGOBI(removeVariableByIndex)(gint which, datad *d, ggobid *gg)
   gint i, j;
   for (i = 0; i < d->nrows; i++) {
    for (j = which+1; j < d->ncols; j++) {
+/*XXX Fill in */
    }
   }
 
@@ -1513,7 +1552,7 @@ GGOBI(removeNumberedKeyEventHandler)(ggobid *gg)
  return(GGOBI(registerNumberedKeyEventHandler(NULL, NULL, NULL, NULL, gg, C)));
 }
 
-#ifdef HAVE_CONFIG_H
+#ifdef HAVE_GGOBI_CONFIG_H
 #include "config.h"
 static gchar *version_date = GGOBI_RELEASE_DATE;
 int GgobiVersionNumbers[] = {MAJOR_VERSION, MINOR_VERSION, PATCH_LEVEL};
@@ -1686,22 +1725,20 @@ GGOBI(getTour2DProjectionMatrix)(gint ncols, gint ndim, gboolean vals_scaled,
   return ((const gdouble **) Fvals);
 }
 
-/* something like this -- dfs --
-void
-GGOBI(edge_menus_update)(ggobid *gg)
+
+guint 
+getGGobiSignal(GGobiSignalType which)
 {
-  GList *dlist;
-  displayd *display;
-  for (dlist = gg->displays; dlist != NULL; dlist = dlist->next) {
-    display = (displayd *) dlist->data;
-    if (GTK_WIDGET_REALIZED (GTK_GGOBI_WINDOW_DISPLAY(display)->window) &&
-        GTK_IS_GGOBI_SCATTERPLOT_DISPLAY(display))
-    {
-      scatterplot_display_edge_menu_update (GTK_GGOBI_DISPLAY(display),
-        gg->app.sp_accel_group, display_options_cb, gg);
-    }
-  }
+  /*
+XXX  assert(which > -1 && which < MAX_GGOBI_SIGNALS);
+  */
+  return(GGobiSignals[which]);
 }
-*/
 
 
+GSList *
+GGOBI(getExtendedDisplayTypes)()
+{
+  return(ExtendedDisplayTypes);
+}
+
