@@ -9,6 +9,7 @@
     it without violating AT&T's intellectual property rights.
 */
 #define GGOBIINTERN
+#define GGOBI_C
 
 #include <math.h>
 #include <stdlib.h>
@@ -21,7 +22,7 @@
 #include "vars.h"
 #include "externs.h"
 
-#ifdef HAVE_CONFIG_H
+#ifdef HAVE_GGOBI_CONFIG_H
 #include "config.h"
 #endif
 
@@ -31,6 +32,7 @@
 #include "colorscheme.h"
 
 #ifdef WIN32
+#include <windows.h>
 #define DIR_SEPARATOR '\\'
 #else
 #define DIR_SEPARATOR '/'
@@ -67,13 +69,12 @@ const gchar * const ViewTypes[] =
 "Parallel Coordinates",
 #endif
 };
-const gint ViewTypeIndices[] = {};
+const gint ViewTypeIndices[];
 
-const gchar *const DataModeNames[num_data_modes] =
-  {"ASCII", "binary", "R/S data", "XML", "MySQL", "URL", "Unknown"};
+FatalErrorHandler FatalError = &exit;
 
+static gchar *computeGGobiHome(char *str);
 
-void initSessionOptions();
 
 gchar *
 getOptValue(const gchar * const name, const gchar * const value)
@@ -114,13 +115,20 @@ parse_command_line (gint *argc, gchar **av)
       showHelp();
       exit(0);
     }
-    else if (strcmp (av[1], "-s") == 0)
+    else if (strcmp (av[1], "-s") == 0) {
       sessionOptions->data_mode = Sprocess_data;
-    else if (strcmp (av[1], "-ascii") == 0) {
+      sessionOptions->data_type = g_strdup(av[1]+1);
+    }
+    else if (strcmp (av[1], "-csv") == 0) {
+      sessionOptions->data_mode = csv_data;
+      sessionOptions->data_type = g_strdup(av[1]+1);
+    } else if (strcmp (av[1], "-ascii") == 0) {
       sessionOptions->data_mode = ascii_data;
+      sessionOptions->data_type = g_strdup(av[1]+1);
     }
     else if (strcmp (av[1], "-xml") == 0) {
       sessionOptions->data_mode = xml_data;
+      sessionOptions->data_type = g_strdup(av[1]+1);
     } else if (strcmp (av[1], "-v") == 0 || strcmp (av[1], "--validate") == 0) {
       extern gint xmlDoValidityCheckingDefaultValue;
       xmlDoValidityCheckingDefaultValue = 1;
@@ -189,9 +197,15 @@ parse_command_line (gint *argc, gchar **av)
       (*argc)--; av++;
     } else if((ptr = getOptValue("restore", av[1]))) {
       sessionOptions->restoreFile = ptr;
+    } else if((ptr = getOptValue("tourSpeed", av[1]))) {
+      sessionOptions->defaultTourSpeed = atof(ptr);
+    } else if((ptr = getOptValue("tour1dSpeed", av[1]))) {
+      sessionOptions->defaultTour1dSpeed = atof(ptr);
     } else if((ptr = getOptValue("plugin", av[1]))) {
-      sessionOptions->pluginFiles = g_slist_append(sessionOptions->pluginFiles,
-        g_strdup(ptr));
+      sessionOptions->pluginFiles = g_slist_append(sessionOptions->pluginFiles, g_strdup(ptr));
+    } else if(strcmp(av[1], "-home") == 0 || strcmp(av[1], "--home") == 0) {
+      fprintf(stdout, "%s\n", computeGGobiHome(av[0]));fflush(stdout);
+      exit(0);
     }
   }
 
@@ -210,15 +224,6 @@ parse_command_line (gint *argc, gchar **av)
   return 1;
 }
 
-gint GGOBI(main)(gint argc, gchar *argv[], gboolean processEvents);
-
-gint 
-main(gint argc, gchar *argv[])
-{ 
- GGOBI(main)(argc, argv, true);
- return (0);
-}
-
 gint
 ggobi_remove (ggobid *gg)
 { 
@@ -271,7 +276,8 @@ ggobi_remove_by_index (ggobid *gg, gint which)
     gg->d = g_slist_remove (gg->d, d);
   }
 
-  g_free (gg);
+  gtk_object_destroy(GTK_OBJECT(gg));
+/*  g_free (gg); */
 
   return (which);
 }
@@ -320,9 +326,11 @@ ggobid* /*XXX should be void. Change when gtk-object setup settles. */
 ggobi_alloc(ggobid *tmp)
 {
   if(tmp == NULL) {
-      /* Should never happen in new GtkObject-based version. */
-      tmp = (ggobid*) g_malloc (sizeof (ggobid));
-      memset (tmp, '\0', sizeof (ggobid)); 
+      /* Should never happen in new GtkObject-based version. 
+	 tmp = (ggobid*) g_malloc (sizeof (ggobid));
+	 memset (tmp, '\0', sizeof (ggobid)); 
+      */
+	tmp = gtk_type_new(GTK_TYPE_GGOBI);
   }
 
   tmp->firsttime = true;
@@ -381,7 +389,7 @@ ggobi_alloc(ggobid *tmp)
   }
   if (!tmp->activeColorScheme) {
     g_printerr ("failed to find color scheme\n");
-    exit(0);
+    (*FatalError)(0);
   } else colorscheme_init (tmp->activeColorScheme);
     /*
      * the number of colors in use will be tested against the
@@ -411,12 +419,17 @@ ggobiInit(int *argc, char **argv[])
     return;
 
   gtk_init (argc, argv);
+
+  initSessionOptions(*argc, *argv);
  
   GTK_TYPE_GGOBI;
   registerDisplayTypes((GtkTypeLoad *) typeLoaders,
 		       sizeof(typeLoaders)/sizeof(typeLoaders)[0]);
+
+  registerDefaultPlugins(sessionOptions->info);
 }
 
+
   /* Available so that we can call this from R
      without any confusion between which main().
    */
@@ -425,9 +438,6 @@ GGOBI(main)(gint argc, gchar *argv[], gboolean processEvents)
 {
   GdkVisual *vis;
   ggobid *gg;
-  initSessionOptions();
-  sessionOptions->cmdArgs = argv;
-  sessionOptions->numArgs = argc;
 
   ggobiInit(&argc, &argv);
 
@@ -435,7 +445,6 @@ GGOBI(main)(gint argc, gchar *argv[], gboolean processEvents)
 
   parse_command_line (&argc, argv);
 
-
 #ifdef SUPPORT_INIT_FILES
   process_initialization_files();
 #endif
@@ -517,15 +526,101 @@ processRestoreFile(const gchar * const fileName, ggobid *gg)
 }
 
 
-void
-initSessionOptions()
+#ifdef WIN32
+gchar *
+getGGobiHomeFromRegistry()
 {
+  HRESULT hr;
+  gchar *tmp;
+  char *key;
+  BYTE *buf;
+  DWORD bufSize, type;
+  HKEY lkey;
+
+  key = "SOFTWARE\\GGobi";
+
+  hr = RegOpenKeyEx(HKEY_LOCAL_MACHINE, key, 0, KEY_ALL_ACCESS, &lkey);
+  if(hr != ERROR_SUCCESS) {
+    fprintf(stderr, "GGobi not in registry: %s (%d)\n", key, (int) hr);fflush(stderr);
+    return(NULL);
+  }
+
+  RegQueryValueEx(lkey, "Home", NULL, NULL, NULL, &bufSize);
+  buf = (BYTE*) g_malloc(bufSize * sizeof(BYTE));
+
+  RegQueryValueEx(lkey, "Home", NULL, &type, buf, &bufSize);
+  RegCloseKey(lkey);
+
+  if(type == REG_SZ) {
+     tmp = (gchar*) buf;
+     fprintf(stderr, "Registry value of GGobi home: %s\n", tmp);
+  }
+
+  return(tmp);
+}
+#endif
+
+
+/*
+ Computes where GGobi directory is located.
+ Ensures that there is a trailing / at the end.
+*/
+static gchar *
+computeGGobiHome(char *str)
+{
+  gchar *dir, *dirPtr, *tmp;
+
+  tmp = getenv("GGOBI_HOME");
+
+#ifdef WIN32
+  if(!tmp || !tmp[0]) 
+    tmp = getGGobiHomeFromRegistry(); 
+#endif
+
+  if(!tmp) {
+    dir = str;
+    dirPtr = strrchr(dir, G_DIR_SEPARATOR);
+
+    if(!dirPtr) {
+     return(g_strdup(""));
+    }
+
+    /*
+    This can't be right, based on the assignment a few characters
+    down.   -- dfs
+    tmp = (char *) g_malloc( ((dirPtr - dir) + 1)*sizeof(char));
+    */
+    tmp = (char *) g_malloc( ((dirPtr - dir) + 2)*sizeof(char));
+    strncpy(tmp, dir, dirPtr-dir + 1);
+    tmp[(dirPtr - dir) + 1] = '\0';
+  }
+
+  if(tmp[strlen(tmp)-1] == G_DIR_SEPARATOR)
+     dir = g_strdup(tmp);
+  else {
+       /* Make certain there is a / at the end of the string. */
+     dir = (char *) g_malloc( (strlen(tmp) + 2)*sizeof(char));
+     sprintf(dir, "%s%c", tmp, G_DIR_SEPARATOR);
+  }
+
+  return(dir);
+}
+
+void
+initSessionOptions(int argc, char **argv)
+{
+  gchar *tmp;
   sessionOptions = &sessionoptions;
   sessionOptions->data_mode = unknown_data;
 
   sessionOptions->showControlPanel = true;
   sessionOptions->verbose = GGOBI_CHATTY;
 
+  sessionOptions->cmdArgs = argv;
+  sessionOptions->numArgs = argc;
+
+  sessionOptions->ggobiHome = computeGGobiHome(argv[0]);
+
 
   sessionOptions->info = (GGobiInitInfo*) g_malloc(sizeof(GGobiInitInfo));
   memset(sessionOptions->info, '\0', sizeof(GGobiInitInfo));
@@ -536,6 +631,14 @@ initSessionOptions()
   sessionOptions->info->numScatMatrixVars = MAXNVARS;
   sessionOptions->info->numParCoordsVars = MAXNPCPLOTS;
   sessionOptions->info->numTimePlotVars = MAXNTSPLOTS;
+
+  tmp = g_malloc(sizeof(gchar) * (strlen(sessionOptions->ggobiHome) + strlen("data/colorschemes.xml") + 2));
+  sprintf(tmp, "%s%s%c%s", sessionOptions->ggobiHome,
+	                   "data", G_DIR_SEPARATOR, "colorschemes.xml");
+  sessionOptions->info->colorSchemeFile = tmp;
+
+  sessionOptions->defaultTourSpeed = 50.0;  
+  sessionOptions->defaultTour1dSpeed = 40.0;  
 }
 
 
@@ -677,8 +780,9 @@ ValidateGGobiRef(ggobid *gg, gboolean fatal)
 
   g_printerr ("Incorrect reference to ggobid.\n");
 
- if (fatal)
-  exit(10);
+  if (fatal) {
+     (*FatalError)(10);
+  }
 
  return(NULL);
 }
@@ -696,7 +800,7 @@ ValidateDatadRef(datad *d, ggobid *gg, gboolean fatal)
   g_printerr("Incorrect reference to datad.\n");
 
  if (fatal)
-  exit(11);
+  (*FatalError)(11);
 
  return(NULL); 
 }
@@ -716,7 +820,7 @@ ValidateDisplayRef(displayd *d, ggobid *gg, gboolean fatal)
   g_printerr("Incorrect reference to display.\n");
 
  if (fatal)
-  exit(11);
+  (*FatalError)(11);
 
  return(NULL); 
 }
@@ -747,17 +851,16 @@ process_initialization_files()
       tmp = getenv("HOME");
       if(tmp) {
         sprintf(buf, "%s/.ggobirc", tmp);
+	if(canRead(buf))
+          fileName = buf;
+        else 
+          fileName = NULL;
+      }
+
+      if(!fileName) {
+        sprintf(buf, "%sggobirc",sessionOptions->ggobiHome);
         fileName = buf;
-      } else {
-        gchar *v;
-        tmp = g_strdup(sessionOptions->cmdArgs[0]);
-        v = strrchr(tmp, DIR_SEPARATOR);
-        if(v) {
-          v[1] = '\0';
-        }
-        sprintf(buf, "%sggobirc",tmp);
-        fileName = buf;
-        g_free(tmp);
+        /*g_free(tmp);*/ /* purify objects to this */
       }
     }
     if(fileName)
@@ -768,6 +871,7 @@ process_initialization_files()
     info = read_init_file(fileName, sessionOptions->info);
     /* sessionOptions->info = info; */
   }
+
   if(sessionOptions->pluginFiles) {
     GSList *el = sessionOptions->pluginFiles;
     while(el) {
@@ -777,6 +881,12 @@ process_initialization_files()
   }
 }
 
+GGobiOptions *
+GGOBI_getSessionOptions()
+{
+  return(sessionOptions);
+}
+
 
 /* This includes code that provides information about the 
    sizes of the data structures in GGobi when it was compiled.
