@@ -13,6 +13,11 @@
 #include "scatterplotClass.h"
 #include "scatmatClass.h"
 
+#ifdef GTK_2_0
+#undef __gtk_marshal_MARSHAL_H__
+#include "marshal.h"
+#endif
+
 
 extern gint num_ggobis, totalNumGGobis;
 extern ggobid **all_ggobis;
@@ -54,6 +59,7 @@ GtkType gtk_ggobi_get_type(void)
  */
 void gtk_ggobi_class_init(GtkGGobiClass * klass)
 {
+#ifndef GTK_2_0
   if (gtk_signal_lookup("datad_added", GTK_TYPE_GGOBI) == 0) {
     GGobiSignals[DATAD_ADDED_SIGNAL] =
       gtk_object_class_user_signal_new(gtk_type_class(GTK_TYPE_GGOBI),
@@ -81,7 +87,11 @@ void gtk_ggobi_class_init(GtkGGobiClass * klass)
       gtk_object_class_user_signal_new(gtk_type_class(GTK_TYPE_GGOBI),
         "move_point",
         GTK_RUN_LAST | GTK_RUN_ACTION,
+#ifdef GTK_2_0
+        gtk_marshal_VOID__POINTER_INT_POINTER,
+#else
         gtk_marshal_NONE__POINTER_INT_POINTER,
+#endif
         GTK_TYPE_NONE, 3,
         GTK_TYPE_GGOBI_SPLOT,
         GTK_TYPE_INT,
@@ -93,7 +103,11 @@ void gtk_ggobi_class_init(GtkGGobiClass * klass)
       gtk_object_class_user_signal_new(gtk_type_class(GTK_TYPE_GGOBI),
         "identify_point",
         GTK_RUN_LAST | GTK_RUN_ACTION,
+#ifdef GTK_2_0
+        gtk_marshal_VOID__POINTER_INT_POINTER,
+#else
         gtk_marshal_NONE__POINTER_INT_POINTER,
+#endif
         GTK_TYPE_NONE, 3,
         GTK_TYPE_GGOBI_SPLOT,
         GTK_TYPE_INT,
@@ -175,6 +189,7 @@ void gtk_ggobi_class_init(GtkGGobiClass * klass)
       GTK_TYPE_NONE, 1, 
       GTK_TYPE_GGOBI_DATA);  /* datad pointer */
   }
+#endif /* GTK_2_0 */
 }
 
 /****************************/
@@ -182,6 +197,7 @@ void gtk_ggobi_class_init(GtkGGobiClass * klass)
 
 void gtk_ggobi_data_class_init(GtkGGobiDataClass * klass)
 {
+#ifndef GTK_2_0
   if (gtk_signal_lookup("rows_in_plot_changed", GTK_TYPE_GGOBI_DATA) == 0) {
     klass->signals[ROWS_IN_PLOT_CHANGED_SIGNAL] = 
     gtk_object_class_user_signal_new(gtk_type_class(GTK_TYPE_GGOBI_DATA), 
@@ -191,6 +207,19 @@ void gtk_ggobi_data_class_init(GtkGGobiDataClass * klass)
         GTK_TYPE_NONE, 3, 
         GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GGOBI);
   }
+#else
+  if (gtk_signal_lookup("rows_in_plot_changed", GTK_TYPE_GGOBI_DATA) == 0) {
+    klass->signals[ROWS_IN_PLOT_CHANGED_SIGNAL] = 
+    g_signal_new(
+        "rows_in_plot_changed",
+	GTK_TYPE_GGOBI_DATA, 
+        GTK_RUN_LAST | GTK_RUN_ACTION, 
+	0, NULL, NULL,
+        gtk_marshal_VOID__INT_INT_POINTER, 
+        GTK_TYPE_NONE, 3, 
+        GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GGOBI);
+  }
+#endif
 }
 
 
@@ -198,6 +227,10 @@ GtkType gtk_ggobi_data_get_type(void)
 {
   static GtkType data_type = 0;
 
+#ifdef DEBUGGING_PLUGINS
+  fprintf(stderr, "gtk_ggobi_data_get_type\n");fflush(stderr);
+#endif
+
   if (!data_type) {
     static const GtkTypeInfo data_info = {
       "GtkGGobiData",
@@ -209,7 +242,9 @@ GtkType gtk_ggobi_data_get_type(void)
       /* reserved_2 */ NULL,
       (GtkClassInitFunc) NULL,
     };
-
+#ifdef DEBUGGIN_PLUGINS
+    fprintf(stderr, "Registering data type\n");fflush(stderr);
+#endif
     data_type = gtk_type_unique(gtk_object_get_type(), &data_info);
   }
 
@@ -280,6 +315,7 @@ gtk_ggobi_display_class_init(GtkGGobiDisplayClass * klass)
   GTK_OBJECT_CLASS(klass)->destroy = testDisplayDestroy;
 #endif
 
+#ifndef GTK_2_0
   if (gtk_signal_lookup("tour_step", GTK_TYPE_GGOBI_DISPLAY) == 0) {
     GGobiSignals[TOUR_STEP_SIGNAL] =
       gtk_object_class_user_signal_new(gtk_type_class(GTK_TYPE_GGOBI_DISPLAY),
@@ -290,6 +326,8 @@ gtk_ggobi_display_class_init(GtkGGobiDisplayClass * klass)
                                          GTK_TYPE_POINTER, GTK_TYPE_INT,
                                          GTK_TYPE_GGOBI);
   }
+#endif
+
 }
 
 static void 
@@ -338,6 +376,12 @@ GtkType gtk_ggobi_display_get_type(void)
 }
 
 
+void
+initWindowDisplayd(windowDisplayd *dpy)
+{
+   GTK_GGOBI_WINDOW_DISPLAY(dpy)->useWindow = true;
+}
+
 GtkType gtk_ggobi_window_display_get_type(void)
 {
   static GtkType data_type = 0;
@@ -348,7 +392,7 @@ GtkType gtk_ggobi_window_display_get_type(void)
       sizeof(struct _windowDisplayd),
       sizeof(GtkGGobiWindowDisplayClass),
       (GtkClassInitFunc) NULL,
-      (GtkObjectInitFunc) NULL,
+      (GtkObjectInitFunc) initWindowDisplayd,
       /* reserved_1 */ NULL,
       /* reserved_2 */ NULL,
       (GtkClassInitFunc) NULL,
@@ -401,6 +445,11 @@ gtk_splot_init(splotd * sp)
 /*sp->tour1d.firsttime = true; *//* Ensure that the 1D tour should be initialized. */
 }
 
+/* In a version of a Gtk on my (DTL) Mac, there is no definition for GTK_TYPE_DRAWING_AREA */
+#ifndef GTK_TYPE_DRAWING_AREA
+#define GTK_TYPE_DRAWING_AREA gtk_drawing_area_get_type
+#endif
+
 static void
 splotDestroy(GtkObject *obj)
 {
@@ -657,7 +706,7 @@ allocCPanels(GTK_GGOBI_EXTENDED_DISPLAY(dpy));
 void allocCPanels(extendedDisplayd * dpy)
 {
   GtkGGobiExtendedDisplayClass *klass;
-  klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(dpy)->klass);
+  klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(dpy));
 
   if (klass->numControlPanels > 0)
     dpy->cpanel =
