@@ -34,13 +34,14 @@ gboolean record_add (eeMode mode, gint a, gint b, gchar *lbl, gchar *id,
 
   /*-- eventually check whether a->b already exists before adding --*/
   if (mode == ADDING_EDGES) {
+
     g_assert (e->edge.n == e->nrows);
     g_assert (a >= 0 && b >= 0 && a != b);
 
     dtarget = e;
   }
 
-  /*-- Make sure the id is unique --*/
+  /*-- Make sure the id is unique -- usually only if adding points --*/
   if (dtarget->idTable && id) {
     gchar *stmp;
     if (id && strlen(id) > 0) stmp = g_strdup (id);
@@ -56,12 +57,6 @@ gboolean record_add (eeMode mode, gint a, gint b, gchar *lbl, gchar *id,
     g_free (stmp);
   }
 
-/*
-  * this may still need tuning, because added records with
-  * categorical variables should have values
-  * that match one of the existing level values, and we're
-  * not testing for that yet.
-*/
   if (dtarget->ncols) {
     raw = (greal *) g_malloc (dtarget->ncols * sizeof(greal));
     for (j=0; j<dtarget->ncols; j++) {
@@ -71,10 +66,39 @@ gboolean record_add (eeMode mode, gint a, gint b, gchar *lbl, gchar *id,
         found_missings = true;
       } else {
         x = (greal) atof (vals[j]);
-        if (vt->vartype == categorical)
-          raw[j] = (greal) floor(x + .5);
-        else
-          raw[j] = x;
+        if (vt->vartype == categorical) {
+	  /* Loop over levels, and add to the one that is closest to x.
+	   * Also, increment vt->level_counts[level] */
+	  gint k, level=0, dist, ddist;
+	  for (k=0; k<vt->nlevels; k++) {
+	    dist = fabs((greal)vt->level_values[k] - x);
+	    if (k == 0) ddist = dist; /* initialize ddist */
+	    else {
+	      if (dist < ddist) {
+		level = k;
+		ddist = dist;
+	      }
+	    }
+	  } 
+	  raw[j] = (greal)vt->level_values[level];
+          vt->level_counts[level]++;
+
+          /* then update the table -- ugh -- no event for this yet.  I
+            should make the expose event repopulate the table ... and
+            I should update the table whenever any cases are added,
+            categorical or not, come to think of it.  nNAs could
+            change, as well as Any of the real variable stats.
+          */
+	  {
+            if (d->vartable_clist[categorical] != NULL) {
+              gint vartable_rownum_from_varno (gint jvar, vartyped vartype, datad *d);
+              gtk_clist_set_text (GTK_CLIST (d->vartable_clist[categorical]),
+	        level+1+vartable_rownum_from_varno (j, vt->vartype, d),
+                CAT_CLIST_LEVEL_COUNT,
+                g_strdup_printf ("%d", vt->level_counts[level]));
+	    }
+          }
+        } else raw[j] = x;
       }
     }
   }
@@ -113,7 +137,10 @@ gboolean record_add (eeMode mode, gint a, gint b, gchar *lbl, gchar *id,
 
   /*-- allocate and initialize brushing arrays --*/
   br_glyph_ids_add (dtarget, gg);
+  /*-- this is adding the brushing color when it should use the color
+    of the points, really --*/
   br_color_ids_add (dtarget, gg);
+  dtarget->color.els[dtarget->nrows-1] = dtarget->color_now.els[dtarget->nrows-1]= d->color.els[a];
   br_hidden_alloc (dtarget);
   vectorb_realloc (&dtarget->pts_under_brush, dtarget->nrows);
   clusters_set (dtarget, gg);
@@ -151,6 +178,15 @@ gboolean record_add (eeMode mode, gint a, gint b, gchar *lbl, gchar *id,
     e->edge.sym_endpoints[dtarget->nrows-1].jpartner = -1; /* XXX */
     unresolveAllEdgePoints(e);
     resolveEdgePoints (e, d);
+    /*
+     * If this is the first edge in the edge set, do something to
+     * make it show up in the display menu.
+     */
+    if (e->nrows == 1) {
+      void GGOBI(edge_menus_update)(ggobid *gg);
+      GGOBI(edge_menus_update)(gg);
+    }
+
   } else {
     GSList *l;
     datad *dd;
@@ -178,7 +214,7 @@ DTL: So need to call unresolveEdgePoints(e, d) to remove it from the
 /*
  * This will be handled with signals, where each splotd listens
  * for (maybe) point_added or edge_added events.
- * New bug: sp->bar needs to be reinitialized.  This code need
+ * New bug: sp->bar needs to be reinitialized.  This code needs
  * to be class-sensitive, and it isn't.
 */
 /* could put some code in splot_record_add  */
@@ -207,7 +243,7 @@ DTL: So need to call unresolveEdgePoints(e, d) to remove it from the
           /*-- this is only necessary if there are variables, I think --*/
           if (GTK_IS_GGOBI_EXTENDED_SPLOT(sp)) {
             GtkGGobiExtendedSPlotClass *klass;
-            klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT(sp)->klass);
+            klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(sp));
             if(klass->alloc_whiskers)
               sp->whiskers = klass->alloc_whiskers(sp->whiskers, sp,
                 d->nrows, d);
@@ -225,6 +261,9 @@ DTL: So need to call unresolveEdgePoints(e, d) to remove it from the
   }
 /*  */
 
+  /* Adding the first edge: This is almost working, but the edge menu
+     isn't being initialized, and the edges don't appear */
+    
 
   displays_tailpipe (FULL, gg);
 
@@ -365,6 +404,30 @@ find_nearest_edge (splotd *sp, displayd *display, ggobid *gg)
         }
       }
     }
+
+    /* if this edge is bidirectional, figure out which segment
+       we really want to highlight.  Use the distance from the
+       mouse to the two endpoints to decide. */
+    if (lineid != -1) {
+      j = endpoints[lineid].jpartner;
+      if (j != -1 && !e->hidden_now.els[j]) {
+          
+        edge_endpoints_get (lineid, &from, &to, d, endpoints, e);
+       
+        a.x = sp->screen[from].x;
+        a.y = sp->screen[from].y;
+        b.x = sp->screen[to].x;
+        b.y = sp->screen[to].y;
+
+	if ((mpos->x - a.x) * (mpos->x - a.x) + 
+            (mpos->y - a.y) * (mpos->y - a.y) >
+            (mpos->x - b.x) * (mpos->x - b.x) +
+	    (mpos->y - b.y) * (mpos->y - b.y))
+        {
+	  lineid = j;
+        }
+      }		       
+    }
   }
   return(lineid);
 }
@@ -516,8 +579,8 @@ pt_screen_to_raw (icoords *screen, gint id, gboolean horiz, gboolean vert,
 }
 
 void
-fetch_default_record_values (gchar **vals, datad *dtarget, displayd *display,
-  ggobid *gg)
+fetch_default_record_values (gchar **vals, datad *dtarget, 
+  displayd *display, ggobid *gg)
 {
   gint j;
   gcoords eps;
@@ -530,12 +593,23 @@ fetch_default_record_values (gchar **vals, datad *dtarget, displayd *display,
       -1, true, true, /* no id, both horiz and vert are true */
       raw, &eps, dtarget, gg->current_splot, gg);
     for (j=0; j<dtarget->ncols; j++) {
-      /*-- if variable j is categorical, make it an integer --*/
-      /* see longer remark in record_add */
       vt = vartable_element_get (j, dtarget);
-      if (vt->vartype == categorical)
-        vals[j] = g_strdup_printf ("%d", (gint) floor(raw[j]+.5));
-      else
+      if (vt->vartype == categorical) {
+	/* Loop over levels, and choose the one that is closest to x. */
+	gint k, level=0, dist, ddist;
+	for (k=0; k<vt->nlevels; k++) {
+	  dist = fabs((greal)vt->level_values[k] - raw[j]);
+	  if (k == 0) ddist = dist; /* initialize ddist */
+	  else {
+	    if (dist < ddist) {
+		level = k;
+		ddist = dist;
+	    }
+	  }
+	} 
+	vals[j] = g_strdup_printf("%d", vt->level_values[level]);
+        /*vals[j] = g_strdup_printf ("%d", (gint) floor(raw[j]+.5));*/
+      } else
         vals[j] = g_strdup_printf ("%g", raw[j]);
     }
     g_free (raw);
