@@ -124,8 +124,8 @@ gg_write_to_statusbar (gchar *message, ggobid *gg)
     /*-- by default, describe the current datad --*/
     datad *d = datad_get_from_notebook (gg->varpanel_ui.notebook, gg);
     if (d) {
-      gchar *msg = g_strdup_printf ("%s: %d x %d",
-        d->name, d->nrows, d->ncols);
+      gchar *msg = g_strdup_printf ("%s: %d x %d  (%s)",
+        d->name, d->nrows, d->ncols, gg->input->fileName);
       gtk_entry_set_text (GTK_ENTRY(entry), msg);
       g_free (msg);
     }
@@ -165,7 +165,7 @@ varpanel_highd (displayd *display)
 
   if(display && GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
      GtkGGobiExtendedDisplayClass *klass;
-     klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+     klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
      if(klass->varpanel_highd)
        highd = klass->varpanel_highd(display);
   }
@@ -253,9 +253,10 @@ viewmode_set (PipelineMode m, ggobid *gg)
         panel = gg->control_panel[gg->viewmode];
       } else {
         GtkGGobiExtendedDisplayClass *klass;
-        klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
-        panel = klass->viewmode_control_box(display, gg->viewmode,
-          &modeName, gg);
+	if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
+          klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
+          panel = klass->viewmode_control_box(display, gg->viewmode, &modeName, gg);
+	}
       }
 
       gtk_frame_set_label (GTK_FRAME (gg->viewmode_frame), modeName);
@@ -278,9 +279,9 @@ viewmode_set (PipelineMode m, ggobid *gg)
    * type is selected.  (For parcoords and scatmat plots, the
    * value of projection is irrelevant.)
   */
-  if (display) {
+  if (display && GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
     GtkGGobiExtendedDisplayClass *klass;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
     if(klass->viewmode_set)
        klass->viewmode_set(display, gg);
   }
@@ -493,7 +494,7 @@ GGOBI(full_viewmode_set)(gint action, ggobid *gg)
 
       procs_activate (on, display, gg);
       if (gg->viewmode != BRUSH && prev_viewmode == BRUSH) {
-        if (cpanel->br_mode == BR_TRANSIENT) {
+        if (cpanel->br.mode == BR_TRANSIENT) {
           reinit_transient_p = true;
           reinit_transient_brushing (display, gg);
         }
@@ -688,6 +689,7 @@ make_ui (ggobid *gg)
 {
   GtkWidget *window;
   GtkWidget *hbox, *vbox, *entry;
+  GtkWidget *basement;
 
   gg->tips = gtk_tooltips_new ();
 
@@ -695,14 +697,11 @@ make_ui (ggobid *gg)
   GGobi_widget_set (window, gg, true);
 
 #ifdef TEST_GGOBI_EVENTS
-  gtk_signal_connect (GTK_OBJECT(gg),
-   "splot_new", test_new_plot_cb, (gpointer) "A new plot");
-  gtk_signal_connect(GTK_OBJECT(gg),
-   "datad_added", test_data_add_cb, NULL);
-  gtk_signal_connect(GTK_OBJECT(gg),
-   "sticky_point_added", test_sticky_points, NULL);
-  gtk_signal_connect(GTK_OBJECT(gg),
-   "sticky_point_removed", test_sticky_points, NULL);
+/*  gtk_signal_connect (GTK_OBJECT(gg), "splot_new", test_new_plot_cb, (gpointer) "A new plot"); */
+  gtk_signal_connect_object(GTK_OBJECT(gg), "splot_new", test_new_plot_cb, (gpointer) "A new plot");
+  gtk_signal_connect(GTK_OBJECT(gg), "datad_added", test_data_add_cb, NULL);
+  gtk_signal_connect(GTK_OBJECT(gg), "sticky_point_added", test_sticky_points, NULL);
+  gtk_signal_connect(GTK_OBJECT(gg), "sticky_point_removed", test_sticky_points, NULL);
 #endif
 
 /*
@@ -716,10 +715,10 @@ make_ui (ggobid *gg)
 /*gtk_window_set_policy (GTK_WINDOW (window), allow_shrink, allow_grow, auto_shrink);*/
   gtk_window_set_policy (GTK_WINDOW (window), false,        true,       false);
 
-  gtk_signal_connect(GTK_OBJECT (window), "delete_event",
-                     GTK_SIGNAL_FUNC (ggobi_close), gg);
-  gtk_signal_connect(GTK_OBJECT (window), "destroy_event",
-                     GTK_SIGNAL_FUNC (ggobi_close), gg); 
+  gtk_signal_connect_object(GTK_OBJECT (window), "delete_event",
+                            GTK_SIGNAL_FUNC (ggobi_close), (gpointer) gg);
+  gtk_signal_connect_object(GTK_OBJECT (window), "destroy_event",
+                            GTK_SIGNAL_FUNC (ggobi_close), (gpointer) gg); 
 
   gtk_container_set_border_width (GTK_CONTAINER (window), 10);
 
@@ -775,12 +774,19 @@ make_ui (ggobid *gg)
 
   /*-- status bar --*/
   entry = gtk_entry_new ();
+  gtk_editable_set_editable(GTK_EDITABLE(entry), false);
   gtk_object_set_data (GTK_OBJECT(gg->main_window), "MAIN:STATUSBAR", entry);
   gtk_box_pack_start (GTK_BOX (vbox), entry, false, false, 0);
   /*--            --*/
 
   gtk_widget_show_all (hbox);
 
+  /* -- do not map or show this widget -- */
+  basement = gtk_vbox_new(false, 0);
+  gtk_widget_set_name (basement, "BASEMENT");
+  gtk_box_pack_start (GTK_BOX (hbox), basement, false, false, 0);
+  /* -- do not map or show this widget -- */
+  
   /*-- at this point, the mode could be NULLMODE, P1PLOT, or XYPLOT --*/
   /*mode_submenus_activate (NULL, gg->viewmode, on, gg);*/
   {
