@@ -23,6 +23,8 @@
 #include "GGobiAPI.h"
 
 #include "externs.h"
+#include "tour.h"
+
 
 gint     getPreviousFiles(const xmlDocPtr doc, GGobiInitInfo *info);
 DataMode getPreviousInput(xmlNode *node, InputDescription *input);
@@ -96,6 +98,7 @@ read_init_file(const gchar *filename, GGobiInitInfo *info)
 #ifndef WIN32
   xmlDoValidityCheckingDefaultValue = oldValiditySetting;
 #endif
+
   /* Causes a crash when started with -init notes/ggobirc,
      but not if there is a -colorschemes filename
      g_free(fileName); 
@@ -130,6 +133,24 @@ getXMLElement(xmlNodePtr node, const char *tagName)
   return(node);
 }
 
+
+static void
+getTourSpeedValue(xmlNodePtr node, const xmlDocPtr doc, const gchar *name, gfloat *value)
+{
+  xmlNodePtr el = getXMLElement(node, name);
+  if(el) {
+      gchar *tmp;
+      gfloat val;
+      tmp =  xmlNodeListGetString(doc, XML_CHILDREN(el), 1);
+      val = atof(tmp);
+      if(val > 0 && val < MAX_TOUR_SPEED)
+	  *value = val;
+      else
+	  g_printerr("Value for %s in preferences file is invalid: %f\n", name, val);
+  }
+}
+
+
 gint
 getPreferences(const xmlDocPtr doc, GGobiInitInfo *info)
 {
@@ -233,6 +254,10 @@ getPreferences(const xmlDocPtr doc, GGobiInitInfo *info)
       info->compress = asNumber(tmp);
   }
 
+
+  getTourSpeedValue(node, doc, "tourSpeed", &sessionOptions->defaultTourSpeed);
+  getTourSpeedValue(node, doc, "tour1dSpeed", &sessionOptions->defaultTourSpeed);
+
   return(0);
 }
 
@@ -512,20 +537,11 @@ static gboolean processXMLPluginNode(xmlNodePtr el, GGobiInitInfo *info, xmlDocP
    return(ans);
 }
 
+
 int
-getPlugins(xmlDocPtr doc, GGobiInitInfo *info, gboolean single)
+processPluginNodes(xmlNode *el, GGobiInitInfo *info, xmlDocPtr doc)
 {
-  xmlNode *node, *el = NULL;
   int count = 0;
-
-  if(single) {
-    el = getXMLDocElement(doc, "plugin");
-  } else {
-    node = getXMLDocElement(doc, "plugins");
-    if(node)
-       el = XML_CHILDREN(node);
-  }
-
   if(el == NULL)  
       return(-1);
 
@@ -534,10 +550,39 @@ getPlugins(xmlDocPtr doc, GGobiInitInfo *info, gboolean single)
           count++;
       el = el->next;
   }
-
   return(count);
 }
 
+int
+getPlugins(xmlDocPtr doc, GGobiInitInfo *info, gboolean single)
+{
+  xmlNode *node, *el = NULL;
+  int count = 0;
+
+  if(single) {
+     /* Call recursively with single = false to get the "plugins" element. 
+        If there are any, then return assuming that we have <plugins> <plugin> ...</plugins>.
+        We will ignore any <plugin> outside of this.
+      */
+    if((count = getPlugins(doc, info, false)) > -1)
+	return(count);
+
+    /* Now handle the stand-alone <plugin> elements. */  
+    el = getXMLDocElement(doc, "plugin");
+    count = processPluginNodes(el, info, doc);
+    el = getXMLDocElement(doc, "inputPlugin");
+    count += processPluginNodes(el, info, doc);
+  } else {
+    node = getXMLDocElement(doc, "plugins");
+    if(node)
+       el = XML_CHILDREN(node);
+  }
+
+  return(processPluginNodes(el, info, doc));
+}
+
+
+
 /*
   This handles the details of a <plugin> tag,
   reading the description, author, etc.
@@ -749,7 +794,7 @@ loadPluginLibrary(GGobiPluginDetails *plugin, GGobiPluginInfo *realPlugin)
     }
   }
 
-  plugin->library = load_plugin_library(plugin);
+  plugin->library = load_plugin_library(plugin, true);
   plugin->loaded = plugin->library != NULL ? DL_LOADED : DL_FAILED;
 
   if(plugin->loaded == DL_LOADED && GGobi_checkPlugin(plugin) && plugin->onLoad) {
@@ -939,9 +984,38 @@ getInputPluginValues(xmlNodePtr node, GGobiInputPluginInfo *plugin,
   c = getXMLElement(node, "modeName");
   if(c) {
     xmlChar *val = xmlNodeListGetString(doc, XML_CHILDREN(c), 1);      
-    plugin->modeName = (char *) val;
+
+    plugin->modeNames = (char **) g_malloc(sizeof(char *));
+    plugin->modeNames[0] = g_strdup(val);
+    plugin->numModeNames = 1;
+  } else if((c = getXMLElement(node, "modeNames"))) {
+	  xmlNodePtr tmp;
+	  int ctr = 0;
+	  tmp = XML_CHILDREN(c);
+	  while(tmp) {
+	     if(strcmp((const char *) tmp->name, "modeName") == 0) {
+		     ctr++;
+	     }
+  	     tmp = tmp->next;
+          }
+	  if(ctr > 0) {
+	     plugin->modeNames = (char **) g_malloc(ctr * sizeof(char *));
+	     plugin->numModeNames = ctr;
+	     ctr = 0;
+	     tmp = XML_CHILDREN(c);
+  	     while(tmp) {
+	       if(strcmp((const char *) tmp->name, "modeName") == 0) {
+		  xmlChar *val = xmlNodeListGetString(doc, XML_CHILDREN(tmp), 1);
+  	          plugin->modeNames[ctr] = g_strdup(val);
+		  ctr++;
+	       }
+     	       tmp = tmp->next;
+             }
+	  }
   }
 
+
+
   c = getXMLElement(node,"dll");
   if(!c)
     return;
@@ -1046,7 +1120,6 @@ GGobiPluginInfo *
 readPluginFile(const char * const fileName, GGobiInitInfo *info)
 {
   xmlDocPtr doc;
-  xmlNodePtr node;
   GGobiPluginInfo *plugin = NULL;
   int n;
 
