@@ -1,8 +1,4 @@
-#if 1
 #include <libxml/parserInternals.h>
-#else
-#include <parserInternals.h>
-#endif
 
 /*
    This is a SAX based parser for reading a single input file
@@ -148,6 +144,37 @@ const gchar * const xmlDataTagNames[] = {
   ""
   };
 
+
+gboolean
+read_xml(InputDescription *desc, ggobid *gg, GGobiPluginInfo *plugin)
+{
+   return(data_xml_read(desc, gg));
+}
+
+InputDescription *
+read_xml_input_description(const char * const fileName, const char * const modeName, 
+			   ggobid *gg, GGobiPluginInfo *info)
+{
+  InputDescription *desc;
+  desc = (InputDescription*) g_malloc(sizeof(InputDescription));
+  memset(desc, '\0', sizeof(InputDescription));
+
+  desc->fileName = g_strdup(fileName);
+  if(canRead(desc->fileName) == false) {
+     g_free(desc->fileName);
+     desc->fileName = g_malloc((strlen(fileName) + 5) * sizeof(gchar));
+     sprintf(desc->fileName, "%s.xml", fileName);
+  }
+
+  desc->mode = isURL(fileName) ? url_data : xml_data;
+  desc->desc_read_input = &read_xml;	
+
+  return(desc);
+}
+
+
+
+
 void
 ggobi_XML_warning_handler(void *data, const gchar *msg, ...)
 {
@@ -556,19 +583,46 @@ setEdgePartners (XMLParserData *parserData)
   datad *e = getCurrentXMLData(parserData);
   SortableEndpoints *ep;
   gint i, k, n;
+  gboolean dups = false;
 
   if (e->edge.n <= 0)
     return;
 
   n = 2 * e->edge.n;
-  ep = (SortableEndpoints *) g_malloc (n * sizeof (SortableEndpoints));
+  ep = (SortableEndpoints *)
+    g_malloc (n * sizeof (SortableEndpoints));
+
+  /*-- Assume that we have no edges from i to i --*/
+
+  /*
+   *  Before setting partners, check for duplicate edges.  If we find
+   *  one, bail out -- we don't handle them correctly, and we don't
+   * want them around.
+  */
 
-  /*-- I'm assuming that we have no edges from i to i --*/
   for (i=0, k=0; i<e->edge.n; i++) {
     ep[k].a = g_strdup ((gchar *) e->edge.sym_endpoints[i].a);
     ep[k].b = g_strdup ((gchar *) e->edge.sym_endpoints[i].b);
     ep[k].jcase = i;
     k++;
+  }
+  qsort ((gchar *) ep, e->edge.n, sizeof (SortableEndpoints), edgecompare);
+
+  for (i=1; i<e->edge.n; i++) {
+    k = i-1;
+    if (strcmp(ep[i].a, ep[k].a) == 0 && strcmp(ep[i].b, ep[k].b) == 0) {
+      g_printerr ("Error: found duplicate edge from %s to %s\n",
+		  (gchar *) e->edge.sym_endpoints[ ep[i].jcase ].a,
+		  (gchar *) e->edge.sym_endpoints[ ep[i].jcase ].b);
+      dups = true;
+    }
+  }
+  if (dups) 
+    /* */ (*FatalError)(1);
+
+
+  /*-- If there are no dups, add the reverse of each edge and re-sort --*/
+  for (i=0, k=e->edge.n; i<e->edge.n; i++) {
     ep[k].a = g_strdup ((gchar *) e->edge.sym_endpoints[i].b);
     ep[k].b = g_strdup ((gchar *) e->edge.sym_endpoints[i].a);
     ep[k].jcase = i;
@@ -576,6 +630,7 @@ setEdgePartners (XMLParserData *parserData)
   }
   qsort ((gchar *) ep, n, sizeof (SortableEndpoints), edgecompare);
 
+
   for (i=1; i<n; i++) {
     k = i-1;
     if (strcmp(ep[i].a, ep[k].a) == 0 && strcmp(ep[i].b, ep[k].b) == 0) {
@@ -603,13 +658,16 @@ void endXMLElement(void *user_data, const xmlChar *name)
     case DATASET:
     {
       datad *d = getCurrentXMLData(data);
+
+      if (data->current_record < d->nrows) {
+        g_printerr ("There are fewer records than declared for '%s': %d < %d.\n",
+          d->name, data->current_record, d->nrows);
+	releaseCurrentDataInfo(data);
+        (*FatalError)(101);
+      }
+
       setEdgePartners(data);
       releaseCurrentDataInfo(data);
-      if (data->current_record < d->nrows) {
-        g_printerr ("There are fewer records than declared for '%s'; exiting.\n",
-          d->name);
-        exit(101);
-      }
     }
     break;
 
@@ -617,6 +675,11 @@ void endXMLElement(void *user_data, const xmlChar *name)
       data->current_record++;
     break;
     case RECORD:
+      /* This processes every element in the record, and it will
+	 become confused and complain if we don't feed d->ncols
+	 elements to it.  I believe it also handles the </record>
+         tag in the case where record elements have been individually
+         tagged, and it does that without confusion. */
       setRecordValues(data, data->recordString, data->recordStringLength);
       data->current_record++;
       resetRecordInfo(data);
@@ -627,11 +690,8 @@ void endXMLElement(void *user_data, const xmlChar *name)
     break;
     case REAL:
     case INTEGER:
-      setRecordValues(data, data->recordString, data->recordStringLength);
-    break;
     case STRING:
-    /* This is the individual setRecordValue(), i.e. not with an 's' at the end.
-    */
+    /* This is the individual setRecordValue(), i.e. with no 's' at the end. */
       setRecordValue((const char *) data->recordString, data->current_data, data);
       data->current_element++;
     break;
@@ -843,7 +903,7 @@ setDatasetInfo (const xmlChar **attrs, XMLParserData *data)
 
   if (tmp == NULL) {
     g_printerr ("No count attribute\n");
-    exit(101);
+    (*FatalError)(101);
   }
 
   d->nrows = strToInteger(tmp);
@@ -1036,7 +1096,7 @@ setGlyph(const xmlChar **attrs, XMLParserData *data, gint i)
     if (value == UNKNOWN_GLYPH) {
      if(tmp[0] < '0' || tmp[0] > '6') {
        g_printerr ("%s is an illegal value for glyphType; it must be on [0,6]\n", tmp);
-       exit(101);
+       (*FatalError)(101);
      }
 
      value = strToInteger(tmp);
@@ -1263,7 +1323,6 @@ setRecordValues (XMLParserData *data, const xmlChar *line, gint len)
      applyRandomUniforms(d, data);
      return(false);
   }
-
   tmp = strtok((gchar*) line, " \t\n");
 
   while (tmp && (tmp < (gchar *) (line + len))) {
@@ -1272,6 +1331,15 @@ setRecordValues (XMLParserData *data, const xmlChar *line, gint len)
     data->current_element++;
     tmp = strtok (NULL, " \t\n");
   }
+  if (data->current_element < d->ncols) {
+    ggobi_XML_error_handler (data, "Not enough elements\n");
+    /*
+    g_printerr ("Record %d has insufficient elements: %d < %d\n",
+		data->current_record, data->current_element,
+		d->ncols);
+    */
+  }
+
 
   applyRandomUniforms(d, data);
   return (true);
@@ -1407,7 +1475,7 @@ allocVariables (const xmlChar **attrs, XMLParserData *data)
 
   if(tmp == NULL) {
     g_printerr ("No count for variables attribute\n");
-    exit(101);
+    (*FatalError)(101);
   }
 
   d->ncols = strToInteger(tmp);
@@ -1569,6 +1637,7 @@ find_xml_file(const gchar *filename, const gchar *dir, ggobid *gg)
  return(name);
 }
 
+/* I suspect this is unused -- dfs */
 /*
   This is reentrant.  
   First we check the size attribute. Then we check the
@@ -1628,6 +1697,7 @@ setColorMap(const xmlChar **attrs, XMLParserData *data)
   return(true);
 }
 
+/* this is certainly unused -- dfs */
 gboolean
 setColormapEntry(const xmlChar **attrs, XMLParserData *data)
 {
@@ -1762,10 +1832,6 @@ xmlParseColorMap(const gchar *fileName, gint size, XMLParserData *data)
 
   g_free(tmp);
 
-/* doesn't use schemes */
-/*
-  return(size == data->gg->ncolors);
-*/
   return (size);
 }
 
@@ -1897,7 +1963,7 @@ readXMLRecord(const xmlChar **attrs, XMLParserData *data)
   if (i == d->nrows) {
     g_printerr ("There are more records than declared for '%s'; exiting.\n",
       d->name);
-    exit(101);
+    (*FatalError)(101);
   }
 
   data->current_element = 0;
