@@ -200,13 +200,13 @@ variableSelect(GtkWidget *w, displayd *display, splotd *sp, gint jvar, gint togg
   switch (cpanel->projection) {
     case P1PLOT:
       redraw = p1d_varsel (sp, jvar, &jvar_prev, toggle, mouse);
-      if (viewmode_get (gg) == BRUSH && cpanel->br_mode == BR_TRANSIENT)
+      if (viewmode_get (gg) == BRUSH && cpanel->br.mode == BR_TRANSIENT)
         reinit_transient_brushing (display, gg);
     break;
     case XYPLOT:
       redraw = xyplot_varsel (sp, jvar, &jvar_prev, toggle, mouse);
       if (redraw)
-        if (viewmode_get (gg) == BRUSH && cpanel->br_mode == BR_TRANSIENT)
+        if (viewmode_get (gg) == BRUSH && cpanel->br.mode == BR_TRANSIENT)
           reinit_transient_brushing (display, gg);
     break;
     case TOUR1D:
@@ -333,6 +333,11 @@ varcircleDraw(displayd *display, gint jvar, GdkPixmap *da_pix, ggobid *gg)
       }
     break;
 
+
+
+
+
+
     /*      } 
       else {
 
@@ -808,10 +813,10 @@ binningPermitted(displayd* dpy)
        cpanel->p1d.type == ASH &&
        cpanel->p1d.ASH_add_lines_p)
      return(false);
-
+  /*
   if (cpanel->br_point_targets == br_select)
      return(false);
-
+  */
   /*-- if we're drawing edges --*/
   if (e != NULL && e->edge.n > 0) {
     if (dpy->options.edges_undirected_show_p ||
@@ -837,20 +842,20 @@ cpanelSet(displayd *dpy, cpaneld *cpanel, ggobid *gg)
       }
 #endif
 
-      cpanel_p1d_set (cpanel, gg);
-      cpanel_xyplot_set (cpanel, gg);
-      cpanel_tour1d_set (cpanel, gg);
+      cpanel_p1d_set (dpy, cpanel, gg);
+      cpanel_xyplot_set (dpy, cpanel, gg);
+      cpanel_tour1d_set (dpy, cpanel, gg);
       if (dpy->d->ncols >= MIN_NVARS_FOR_TOUR2D3)
-        cpanel_tour2d3_set (cpanel, gg);
+        cpanel_tour2d3_set (dpy, cpanel, gg);
       if (dpy->d->ncols >= MIN_NVARS_FOR_TOUR2D)
-        cpanel_tour2d_set (cpanel, gg);
+        cpanel_tour2d_set (dpy, cpanel, gg);
       if (dpy->d->ncols >= MIN_NVARS_FOR_COTOUR)
-        cpanel_tourcorr_set (cpanel, gg);
+        cpanel_tourcorr_set (dpy, cpanel, gg);
 
-      cpanel_brush_set (cpanel, gg);
-      cpanel_scale_set (cpanel, gg);
-      cpanel_edgeedit_set (cpanel, gg);
-      cpanel_identify_set (cpanel, gg);
+      cpanel_brush_set (dpy, cpanel, gg);
+      cpanel_scale_set (dpy, cpanel, gg);
+      cpanel_edgeedit_set (dpy, cpanel, gg);
+      cpanel_identify_set (dpy, cpanel, gg);
 
       return(true);
 }
@@ -1171,8 +1176,10 @@ worldToPlane(splotd *sp, datad *d, ggobid *gg)
 static gboolean
 drawCase(splotd *sp, gint m, datad *d, ggobid *gg)
 {
+  displayd *display = sp->displayptr;
   gboolean draw_case = true;
   gint proj = projection_get (gg);
+  gint j;
 
   switch (proj) {
     case P1PLOT:
@@ -1186,23 +1193,45 @@ drawCase(splotd *sp, gint m, datad *d, ggobid *gg)
         draw_case = false;
     break;
     case TOUR1D:
-      if (d->missing.vals[m][sp->displayptr->t1d.active_vars.els[m]])
-        draw_case = false;
+      for (j=0; j<display->t1d.nactive; j++) {
+        if (d->missing.vals[m][display->t1d.active_vars.els[j]]) {
+          draw_case = false;
+          break;
+        }
+      }
     break;
     case TOUR2D3:
-      if (d->missing.vals[m][sp->displayptr->t2d3.active_vars.els[m]])
-        draw_case = false;
+      for (j=0; j<display->t2d3.nactive; j++) {
+        if (d->missing.vals[m][display->t2d3.active_vars.els[j]]) {
+          draw_case = false;
+          break;
+        }
+      }
     break;
     case TOUR2D:
-      if (d->missing.vals[m][sp->displayptr->t2d.active_vars.els[m]])
-        draw_case = false;
+      for (j=0; j<display->t2d.nactive; j++) {
+        if (d->missing.vals[m][display->t2d.active_vars.els[j]]) {
+          draw_case = false;
+          break;
+        }
+      }
     break;
 
     case COTOUR:
-      if (d->missing.vals[m][sp->displayptr->tcorr1.active_vars.els[m]])
-        draw_case = false;
-      else if (d->missing.vals[m][sp->displayptr->tcorr2.active_vars.els[m]])
-        draw_case = false;
+      for (j=0; j<display->tcorr1.nactive; j++) {
+        if (d->missing.vals[m][display->tcorr1.active_vars.els[j]]) {
+          draw_case = false;
+          break;
+        }
+      }
+      if (draw_case) {
+        for (j=0; j<display->tcorr2.nactive; j++) {
+          if (d->missing.vals[m][display->tcorr2.active_vars.els[j]]) {
+            draw_case = false;
+            break;
+          }
+	}
+      }
     break;
   }
 
@@ -1401,7 +1430,7 @@ scatterplotDisplayClassInit(GtkGGobiScatterplotDisplayClass *klass)
   klass->parent_class.varpanel_tooltips_set = varpanelTooltipsReset;
   klass->parent_class.plotted_vars_get = plottedVarsGet;
 
-  klass->parent_class.titleLabel = "scatterplot display";
+  klass->parent_class.titleLabel = "Scatterplot Display";
   klass->parent_class.treeLabel = "Scatterplot";
   klass->parent_class.ruler_ranges_set = ruler_ranges_set;
   klass->parent_class.xml_describe = add_xml_scatterplot_variables;
