@@ -131,7 +131,7 @@ splot_event_handled (GtkWidget *w, GdkEventKey *event,
 
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
     GtkGGobiExtendedDisplayClass *klass;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
     if(klass->splot_key_event_handler) 
          action = klass->splot_key_event_handler(display, sp, event->keyval);
   }
@@ -239,7 +239,7 @@ sp_event_handlers_toggle (splotd *sp, gboolean state)
 
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
     GtkGGobiExtendedDisplayClass *klass;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
     if(klass->event_handlers_toggle && klass->event_handlers_toggle(display, sp, state, m) == false) {
       return;
     }
@@ -382,7 +382,7 @@ GGOBI(splot_set_current_full)(displayd *display, splotd *sp, ggobid *gg)
     */
     if (prev_viewmode == NULLMODE || cpanel == NULL)
       displays_plot (NULL, FULL, gg);
-    if (prev_viewmode == BRUSH && cpanel->br_mode == BR_TRANSIENT)
+    if (prev_viewmode == BRUSH && cpanel->br.mode == BR_TRANSIENT)
       displays_plot (NULL, FULL, gg);
     else if (prev_viewmode == IDENT)
       displays_plot (NULL, QUICK, gg);
@@ -462,7 +462,7 @@ splot_alloc (splotd *sp, displayd *display, ggobid *gg)
 
   if(GTK_IS_GGOBI_EXTENDED_SPLOT(sp)) {
     GtkGGobiExtendedSPlotClass *klass;
-    klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT(sp)->klass);
+    klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(sp));
     if(klass->alloc_whiskers)
       sp->whiskers = klass->alloc_whiskers(sp->whiskers, sp, nr, d);
   }
@@ -511,6 +511,19 @@ splot_new (displayd *display, gint width, gint height, ggobid *gg)
   return(sp);
 }
 
+
+/* 
+  Key action callback to raise the control panel window.
+ */
+void
+raise_control_panel(GtkWidget *w, GdkEventKey *ev, ggobid *gg)
+{
+    /* Alt-M for "Main" */
+    if(ev->type == GDK_KEY_PRESS && 
+      (ev->state & GDK_MOD1_MASK) && ev->keyval == GDK_m)
+	gdk_window_raise(gg->main_window->window);
+}
+
 void
 splot_init(splotd *sp, displayd *display, gint width, gint height, ggobid *gg) 
 {
@@ -588,9 +601,15 @@ splot_init(splotd *sp, displayd *display, gint width, gint height, ggobid *gg)
   sp->win32.npoints = 0;
 #endif
 
+  gtk_signal_connect (GTK_OBJECT (GTK_GGOBI_WINDOW_DISPLAY(display)->window),
+		      "key_press_event",
+		      (GtkSignalFunc) raise_control_panel,
+		      (gpointer) gg);
+
   gtk_signal_emit(GTK_OBJECT(gg), GGobiSignals[SPLOT_NEW_SIGNAL], sp);
 }
 
+
 void
 splot_get_dimensions (splotd *sp, gint *width, gint *height) {
   *width = sp->da->allocation.width;
@@ -620,7 +639,7 @@ splot_world_to_plane (cpaneld *cpanel, splotd *sp, ggobid *gg)
 */
 
   if(GTK_IS_GGOBI_EXTENDED_SPLOT(sp)) {
-    GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT(sp)->klass)->world_to_plane(sp,
+    GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(sp))->world_to_plane(sp,
       d, gg);
   }
 }
@@ -642,7 +661,7 @@ splot_plane_to_screen (displayd *display, cpaneld *cpanel, splotd *sp,
   greal precis = (greal) PRECISION1;
 
   if(GTK_IS_GGOBI_EXTENDED_SPLOT(sp)) {
-     klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT(sp)->klass);
+     klass = GTK_GGOBI_EXTENDED_SPLOT_CLASS(GTK_OBJECT_GET_CLASS(sp));
     
      if(klass->plane_to_screen) {
         klass->plane_to_screen(sp, d, gg);
@@ -700,8 +719,9 @@ splot_plane_to_screen (displayd *display, cpaneld *cpanel, splotd *sp,
  * run a pair of points through the reverse pipeline -- but
  * without having any impact on the pipeline data structures.
  *
- * I'm not going to replace this with pt_screen_to_raw, at
- * least not yet.  -- dfs
+ * This should probably be replaced with pt_screen_to_raw
+ * in lineedit.c.  I just have to make sure the results are
+ * consistent. -- dfs
 */
 void
 splot_screen_to_tform (cpaneld *cpanel, splotd *sp, icoords *scr,
@@ -722,6 +742,7 @@ splot_screen_to_tform (cpaneld *cpanel, splotd *sp, icoords *scr,
                     cpanel->projection == TOUR2D ||
                     cpanel->projection == COTOUR);
 
+
   scale_x = sp->scale.x;
   scale_y = sp->scale.y;
   scale_x /= 2;
@@ -798,12 +819,7 @@ splot_screen_to_tform (cpaneld *cpanel, splotd *sp, icoords *scr,
  * The remainder of the reverse pipeline routines operate on
  * the ggobi data structures.
 */
-/*
- * Currently, those routines live in lineedit.c
-*/
 
-
-/*
 void
 splot_screen_to_plane (splotd *sp, gint pt, gcoords *eps,
   gboolean horiz, gboolean vert)
@@ -839,7 +855,6 @@ splot_screen_to_plane (splotd *sp, gint pt, gcoords *eps,
     eps->y = sp->planar[pt].y - prev_planar.y;
   }
 }
-*/
 
 /*
 void
