@@ -15,7 +15,9 @@
 #endif
 
 #include <math.h>
+#ifndef Darwin
 #include <malloc.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -45,6 +47,9 @@ extern gdouble erf (gdouble);  /*-- not defined on all unixes --*/
 #define T1DON true
 #define T1DOFF false
 
+static void tour1d_speed_set_display(gfloat slidepos, displayd *dsp); 
+
+
 #ifdef TESTING_TOUR_STEP
 void tour1d_step_cb(displayd *dsp, tour td, gint projdim, ggobid *gg,
   void *display)
@@ -161,7 +166,7 @@ tour1d_realloc_up (gint nc, datad *d, ggobid *gg)
 
     if(!GTK_IS_GGOBI_EXTENDED_DISPLAY(dsp))
       continue;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(dsp)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(dsp));
     if(klass->tour1d_realloc)
         klass->tour1d_realloc(dsp, nc, d);
   }
@@ -258,6 +263,8 @@ display_tour1d_init (displayd *dsp, ggobid *gg)
 
   dsp->t1d_pp_op.temp_start = 1.0;
   dsp->t1d_pp_op.cooling = 0.99;
+
+  tour1d_speed_set_display(sessionOptions->defaultTour1dSpeed, dsp);
 }
 
 /*-- called from the Options menu --*/
@@ -306,8 +313,9 @@ tour1d_all_vars_cb (GtkCheckMenuItem *w, guint action)
   }
 }
 
-void tour1d_speed_set(gfloat slidepos, ggobid *gg) {
-  displayd *dsp = gg->current_display; 
+static void 
+tour1d_speed_set_display(gfloat slidepos, displayd *dsp) 
+{
   cpaneld *cpanel = &dsp->cpanel;
 
   cpanel->t1d.slidepos = slidepos;
@@ -315,6 +323,11 @@ void tour1d_speed_set(gfloat slidepos, ggobid *gg) {
 
 }
 
+void tour1d_speed_set(gfloat slidepos, ggobid *gg) 
+{
+    tour1d_speed_set_display(slidepos, gg->current_display); 
+}
+
 #ifdef TESTING_TOUR_STEP
 void tour1d_step_cb(displayd *dsp, tour td, gint projdim, ggobid *gg,
   void *display)
@@ -761,7 +774,7 @@ g_printerr ("\n");*/
 #ifdef TESTING_TOUR_STEP
 {
   GtkGGobiDisplayClass *klass;
-  klass = GTK_GGOBI_DISPLAY_CLASS(GTK_OBJECT(dsp)->klass);
+  klass = GTK_GGOBI_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(dsp));
   gtk_signal_emit(GTK_OBJECT(dsp),
     klass->signals[TOUR_STEP_SIGNAL], dsp->t1d,
     (gint) 1, gg);
