@@ -28,7 +28,9 @@ The authors can be contacted at the following email addresses:
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#ifndef Darwin
 #include <malloc.h>
+#endif
 
 #include "vars.h"
 #include "externs.h"
@@ -731,12 +733,12 @@ projection.
 *********************************************************************/
 
 gfloat t1d_calc_indx (array_f pd, 
-                gint (*index) (array_f*, void*, gfloat*),
+                Tour_PPIndex_f index,
                 void *param)
 { 
   gfloat indexval;
 
-  index (&pd, param, &indexval);
+  index (&pd, param, &indexval, NULL);
 
   return(indexval);
 }
@@ -798,9 +800,9 @@ gboolean t1d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
       break;
     case PCA: 
       dsp->t1d.ppval = t1d_calc_indx (dsp->t1d_pp_op.pdata, 
-        pca, NULL);
+        (Tour_PPIndex_f) pca, NULL);
       if (basismeth == 1)
-        kout = optimize0 (&dsp->t1d_pp_op, pca, NULL);
+        kout = optimize0 (&dsp->t1d_pp_op, (Tour_PPIndex_f) pca, NULL);
       break;
     case LDA:
       if (!compute_groups (dsp->t1d_pp_param.group, dsp->t1d_pp_param.ngroup, 
