@@ -41,17 +41,17 @@ static void close_menuitem_cb (displayd *dsp, gint action, GtkWidget *w) {
                                just gets hidden, so shouldn't
                                free the arrays. */
 }
-/*-- called when closed from the window manager --*/
+/*-- called when destroyed from the window manager --*/
 static void
 close_wmgr_cb (GtkWidget *w, GdkEventButton *event, displayd *dsp) {
   gtk_widget_hide (dsp->t1d_window);
   t1d_optimz(0, &dsp->t1d.get_new_target, 
     &dsp->t1d.target_selection_method, dsp);
 
-  /*  free_optimize0_p(&dsp->t1d_pp_op); * should this go here? *
-  free_pp(&dsp->t1d_pp_param); seems not, causes a crash because window
-                               just gets hidden, so shouldn't
-                               free the arrays. */
+  free_optimize0_p(&dsp->t1d_pp_op);
+  free_pp(&dsp->t1d_pp_param);
+  gtk_widget_destroy (dsp->t1d_window);
+  dsp->t1d_window = NULL;
 }
 
 static void
@@ -292,7 +292,7 @@ tour1dpp_window_open (ggobid *gg) {
                         GTK_SIGNAL_FUNC (close_wmgr_cb), (gpointer) dsp);
     /*gtk_window_set_policy (GTK_WINDOW (dsp->t1d_window), true, true, false);*/
     gtk_signal_connect (GTK_OBJECT(d), "rows_in_plot_changed",
-      reset_pp, gg);
+      GTK_SIGNAL_FUNC(reset_pp), gg);
 
     gtk_container_set_border_width (GTK_CONTAINER (dsp->t1d_window), 10);
 
