@@ -17,6 +17,7 @@
 
 #include "vars.h"
 #include "externs.h"
+#include "tour.h"
 
 /* This function initializes the tour variables - it should only be
    called more than once, when a new tour is started since a new
@@ -33,8 +34,7 @@ cpanel_t1d_init (cpaneld *cpanel, ggobid *gg) {
   cpanel->t1d.pp_indx = 0;
 
   cpanel->t1d.ASH_add_lines_p = false;
-  cpanel->t1d.slidepos = 50.;/* If this is changed, it needs to be 
-     changed in th cpanel_tour1d_make routine also. */
+  cpanel->t1d.slidepos = sessionOptions->defaultTour1dSpeed;
   cpanel->t1d.ASH_smooth = 0.19;
 }
 
@@ -48,7 +48,7 @@ ASH_add_lines_cb (GtkToggleButton *button, ggobid *gg)
 
 /*-- scatterplot only; need a different routine for parcoords --*/
 void
-cpanel_tour1d_set (cpaneld *cpanel, ggobid* gg)
+cpanel_tour1d_set (displayd *display, cpaneld *cpanel, ggobid* gg)
 /*
  * To handle the case where there are multiple scatterplots
  * which may have different tour options and parameters selected
@@ -121,7 +121,8 @@ static void t1d_ash_sm_cb (GtkAdjustment *adj, ggobid *gg)
   display_tailpipe (gg->current_display, FULL, gg);
 
   /* Reinits the vertical height for the ashes */
-  sp->tour1d.initmax = true;
+  if(sp)
+    sp->tour1d.initmax = true;
 }
 
 /*
@@ -157,7 +158,7 @@ cpanel_tour1d_make (ggobid *gg) {
   /* Note that the page_size value only makes a difference for
    * scrollbar widgets, and the highest value you'll get is actually
    * (upper - page_size). */
-  adj = gtk_adjustment_new (40.0, 0.0, 100.0, 1.0, 1.0, 0.0);
+  adj = gtk_adjustment_new (sessionOptions->defaultTour1dSpeed, 0.0, MAX_TOUR_SPEED, 1.0, 1.0, 0.0);
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (speed1d_set_cb), (gpointer) gg);
 
@@ -381,7 +382,8 @@ static void hide_cb (GtkWidget *w ) {
 
 #ifdef TOUR_ADV_IMPLEMENTED
 static GtkWidget *window = NULL; /* Note to self - this should be removed */
-static void tour1dadv_window_open (ggobid *gg) {
+static void tour1dadv_window_open (ggobid *gg) 
+{
   GtkWidget *vbox, *box, *btn, *opt, *tgl, *entry;
   GtkWidget *pathlen_opt, *vb, *hb, *lbl, *sbar, *notebook;
   GtkObject *adj;
