@@ -21,7 +21,9 @@ extern gint _finite (gdouble);
 #endif
 
 #include <math.h>
+#ifndef Darwin
 #include <malloc.h>
+#endif
 #include <stdlib.h>
 #include <unistd.h>
 
@@ -45,6 +47,8 @@ extern gdouble erf (gdouble);  /*-- not defined on all unixes --*/
 #define T2DON true
 #define T2DOFF false
 
+static void tour2d_speed_set_display(gfloat slidepos, displayd *dsp);
+
 void
 display_tour2d_init_null (displayd *dsp, ggobid *gg)
 {
@@ -111,7 +115,6 @@ alloc_tour2d (displayd *dsp, ggobid *gg)
   arrayd_alloc(&dsp->t2d_Rmat2, 3, 3);
   arrayd_alloc(&dsp->t2d_mvar_3dbasis, 3, 3);
   arrayd_alloc(&dsp->t2d_manbasis, 3, nc);
-
 }
 
 /*-- eliminate the nc columns contained in *cols --*/
@@ -161,7 +164,7 @@ tour2d_realloc_up (gint nc, datad *d, ggobid *gg)
 
     if(!GTK_IS_GGOBI_EXTENDED_DISPLAY(dsp))
       continue;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(dsp)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(dsp));
     if(klass->tour2d_realloc)
         klass->tour2d_realloc(dsp, nc, d);
   }
@@ -267,6 +270,8 @@ display_tour2d_init (displayd *dsp, ggobid *gg) {
   dsp->t2d_axes = true;
   dsp->t2d_pp_op.temp_start = 1.0;
   dsp->t2d_pp_op.cooling = 0.99;
+
+  tour2d_speed_set_display(sessionOptions->defaultTourSpeed, dsp);
 }
 
 /*-- called from the Options menu --*/
@@ -316,9 +321,14 @@ tour2d_all_vars_cb (GtkCheckMenuItem *w, guint action)
 }
 
 void tour2d_speed_set(gfloat slidepos, ggobid *gg) {
-  cpaneld *cpanel;
-  displayd *dsp = gg->current_display;
 
+  displayd *dsp = gg->current_display;
+  tour2d_speed_set_display(slidepos, dsp);
+}
+
+static void tour2d_speed_set_display(gfloat slidepos, displayd *dsp) 
+{
+  cpaneld *cpanel;
   if (dsp != NULL)
     cpanel = &dsp->cpanel;
   if (cpanel == NULL)
@@ -612,8 +622,7 @@ tour2d_run(displayd *dsp, ggobid *gg)
       /*    if (dsp->t2d_ppda != NULL) {*/
 
       dsp->t2d.oppval = dsp->t2d.ppval;
-      revert_random = t2d_switch_index(cpanel->t2d.pp_indx,
-        0, dsp, gg);
+      revert_random = t2d_switch_index(cpanel->t2d, 0, dsp, gg);
       t2d_ppdraw(dsp->t2d.ppval, dsp, gg);
     }
   }
@@ -685,7 +694,7 @@ tour2d_run(displayd *dsp, ggobid *gg)
         dsp->t2d.oppval = -1.0;
         t2d_ppdraw_think(dsp, gg);
 /*XX*/  gdk_flush ();
-        revert_random = t2d_switch_index(cpanel->t2d.pp_indx, 
+        revert_random = t2d_switch_index(cpanel->t2d, 
           dsp->t2d.target_selection_method, dsp, gg);
 
         if (!revert_random) {
@@ -715,7 +724,7 @@ g_printerr ("\n");
               for (j=0; j<dsp->t2d.nactive; j++)
                 dsp->t2d_pp_op.proj_best.vals[i][j] = 
                   dsp->t2d.Fz.vals[i][dsp->t2d.active_vars.els[j]];
-            revert_random = t2d_switch_index(cpanel->t2d.pp_indx, 
+            revert_random = t2d_switch_index(cpanel->t2d, 
               dsp->t2d.target_selection_method, dsp, gg);
 	      }*/
   /*          t2d_ppdraw(dsp->t2d.ppval, dsp, gg);*/
@@ -1303,7 +1312,7 @@ tour2d_manip(gint p1, gint p2, splotd *sp, ggobid *gg)
       /*    if (dsp->t2d_ppda != NULL) {*/
 
       dsp->t2d.oppval = dsp->t2d.ppval;
-      pp_problem = t2d_switch_index(cpanel->t2d.pp_indx, 
+      pp_problem = t2d_switch_index(cpanel->t2d, 
         0, dsp, gg);
       t2d_ppdraw(dsp->t2d.ppval, dsp, gg);
     }
