@@ -28,7 +28,9 @@ The authors can be contacted at the following email addresses:
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#ifndef Darwin
 #include <malloc.h>
+#endif
 
 #include "vars.h"
 #include "externs.h"
@@ -36,11 +38,9 @@ The authors can be contacted at the following email addresses:
 #include "tour2d_pp.h"
 #include "tour_pp.h"
 
-#define HOLES 0
-#define CENTRAL_MASS 1
-#define LDA 2
-#define CGINI 3
-#define CENTROPY 4
+
+typedef enum {HOLES, CENTRAL_MASS, LDA, CGINI, CENTROPY}  StandardPPIndexTypes;
+
 
 #define EXPMINUS1 0.3678794411714423
 #define ONEMINUSEXPMINUS1 0.63212056
@@ -596,13 +596,13 @@ void t2d_pp_reinit(displayd *dsp, ggobid *gg)
   for (i=0; i<dsp->t2d_pp_op.proj_best.nrows; i++)
     for (j=0; j<dsp->t2d_pp_op.proj_best.ncols; j++)
       dsp->t2d_pp_op.proj_best.vals[i][j] = 
-      dsp->t2d.F.vals[i][dsp->t2d.active_vars.els[j]];
+	  dsp->t2d.F.vals[i][dsp->t2d.active_vars.els[j]];
   dsp->t2d.ppval = 0.0;
   dsp->t2d.oppval = -1.0;
   dsp->t2d_pp_op.index_best = 0.0;
   label = g_strdup_printf ("PP index: (%3.1f) %5.3f (%3.1f)",
-  dsp->t2d_indx_min, dsp->t2d_ppindx_mat[dsp->t2d_ppindx_count], 
-  dsp->t2d_indx_max);
+			   dsp->t2d_indx_min, dsp->t2d_ppindx_mat[dsp->t2d_ppindx_count], 
+			   dsp->t2d_indx_max);
   gtk_label_set_text(GTK_LABEL(dsp->t2d_pplabel),label);
 
   t2d_clear_ppda(dsp, gg);
@@ -631,17 +631,22 @@ projection.
 *********************************************************************/
 
 gfloat t2d_calc_indx (array_f pd,
-                gint (*index) (array_f*, void*, gfloat*),
+                Tour_PPIndex_f index,
                 void *param)
 { 
   gfloat indexval;
 
-  index (&pd, param, &indexval);
+  index (&pd, param, &indexval, NULL);
 
   return(indexval);
 }
 
-gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
+/* 
+ We could call this with the tour t2d object rather than the index type
+ since we no longer need the indxtype 
+*/
+gboolean
+t2d_switch_index(Tour2DCPanel controls, gint basismeth, displayd *dsp,
   ggobid *gg)
 {
   datad *d = dsp->d;
@@ -649,7 +654,7 @@ gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
   gfloat *gdata;
   gint i, j, k;
 
-  if (d->nrows_in_plot <= 1)  /* can't do pp on no data! */
+  if (d->nrows_in_plot == 1)  /* can't do pp on no data! */
     return(false);
 
   /* Copy data into pp opt'n data */
@@ -677,7 +682,7 @@ gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
 
   gdata  = g_malloc (nrows*sizeof(gfloat));
   if (d->clusterid.els==NULL) printf ("No cluster information found\n");
-  for (i=0; i<nrows; i++)
+  for (i = 0 ; i < nrows; i++)
   { 
     if (d->clusterid.els!=NULL)
       gdata[i] = d->clusterid.els[d->rows_in_plot.els[i]];
@@ -685,27 +690,37 @@ gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
       gdata[i] = 0;
   }
 
+  if(controls.ppindex.index_f) {
+      if(controls.ppindex.checkGroups == false || 
+           !compute_groups (dsp->t2d_pp_param.group, dsp->t2d_pp_param.ngroup, &dsp->t2d_pp_param.numgroups, nrows, gdata)) 
+      { 
+	  controls.ppindex.index_f(&dsp->t2d_pp_op.pdata, &dsp->t2d_pp_param,  &dsp->t2d.ppval, controls.ppindex.userData);
+	  if(basismeth == 1) 
+	      kout = optimize0 (&dsp->t2d_pp_op, controls.ppindex.index_f, &dsp->t2d_pp_param);
+      }
+  }
+
+#if 0
   switch (indxtype)
   { 
     case HOLES: 
       dsp->t2d.ppval = t2d_calc_indx (dsp->t2d_pp_op.pdata, 
-        holes_raw, &dsp->t2d_pp_param);
+				      holes_raw, &dsp->t2d_pp_param);
       if (basismeth == 1) {
         kout = optimize0 (&dsp->t2d_pp_op, holes_raw, &dsp->t2d_pp_param);
       }
     break;
     case CENTRAL_MASS: 
       dsp->t2d.ppval = t2d_calc_indx (dsp->t2d_pp_op.pdata,
-        central_mass_raw, &dsp->t2d_pp_param);
+				      central_mass_raw, &dsp->t2d_pp_param);
       if (basismeth == 1)
-        kout = optimize0 (&dsp->t2d_pp_op, central_mass_raw, 
-          &dsp->t2d_pp_param);
+        kout = optimize0 (&dsp->t2d_pp_op, central_mass_raw, &dsp->t2d_pp_param);
     break;
     case LDA: 
       if (!compute_groups (dsp->t2d_pp_param.group, dsp->t2d_pp_param.ngroup, 
         &dsp->t2d_pp_param.numgroups, nrows, gdata)) {
         dsp->t2d.ppval = t2d_calc_indx (dsp->t2d_pp_op.pdata,
-          discriminant, &dsp->t2d_pp_param);
+					discriminant, &dsp->t2d_pp_param);
         if (basismeth == 1)
           kout = optimize0 (&dsp->t2d_pp_op, discriminant, &dsp->t2d_pp_param);
       }
@@ -715,7 +730,7 @@ gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
         &dsp->t2d_pp_param.numgroups, nrows, 
 			   gdata)) {
         dsp->t2d.ppval = t2d_calc_indx (dsp->t2d_pp_op.pdata,
-          cartgini, &dsp->t2d_pp_param);
+					cartgini, &dsp->t2d_pp_param);
         if (basismeth == 1)
           kout = optimize0 (&dsp->t2d_pp_op, cartgini, &dsp->t2d_pp_param);
       }
@@ -732,18 +747,16 @@ gboolean t2d_switch_index(gint indxtype, gint basismeth, displayd *dsp,
       break;
     break;
     default: 
+/* Shouldn't we free gdata. */
       return(true);
     break;
   }
+#endif
+
   g_free (gdata);
   return(false);
 }
 
-#undef HOLES
-#undef CENTRAL_MASS
-#undef LDA
-#undef CGINI      
-#undef CENTROPY   
 
 #undef ONEMINUSEXPMINUS1
 #undef EXPMINUS1
