@@ -17,6 +17,10 @@
 
 #include "vars.h"
 #include "externs.h"
+#include "tour.h"
+
+extern TourPPIndex StandardPPIndices[];
+
 
 /* This function initializes the tour variables - it should only be
    called more than once, when a new tour is started since a new
@@ -31,16 +35,17 @@ cpanel_t2d_init (cpaneld *cpanel, ggobid *gg) {
   cpanel->t2d.step = TOURSTEP0;
   cpanel->t2d.ls_dir = TOUR_LS_IN;
   cpanel->t2d.path_len = 1.;
-  cpanel->t2d.slidepos = 50.;/* If this is changed, it needs to be 
+  cpanel->t2d.slidepos = sessionOptions->defaultTourSpeed;/* If this is changed, it needs to be 
      changed in th cpanel_tour2d_make routine also. */
 
   cpanel->t2d.manip_mode = MANIP_OBLIQUE;
 
-  cpanel->t2d.pp_indx = 0;
+  cpanel->t2d.pp_indx = 0; /* Corresponds to the HOLES */
+  cpanel->t2d.ppindex = StandardPPIndices[0];
 }
 
 void
-cpanel_tour2d_set (cpaneld *cpanel, ggobid* gg)
+cpanel_tour2d_set (displayd *display, cpaneld *cpanel, ggobid* gg)
 /*
  * To handle the case where there are multiple scatterplots
  * which may have different tour options and parameters selected
@@ -145,7 +150,7 @@ cpanel_tour2d_make (ggobid *gg) {
   /* Note that the page_size value only makes a difference for
    * scrollbar widgets, and the highest value you'll get is actually
    * (upper - page_size). */
-  adj = gtk_adjustment_new (50.0, 0.0, 100.0, 1.0, 1.0, 0.0);
+  adj = gtk_adjustment_new (sessionOptions->defaultTourSpeed, 0.0, MAX_TOUR_SPEED, 1.0, 1.0, 0.0);
 
   gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                       GTK_SIGNAL_FUNC (speed2d_set_cb), (gpointer) gg);
