@@ -43,8 +43,9 @@ CreateMenuItem (GtkWidget *menu,
   /* --- If there's a name, create the item and add the signal handler --- */
   if (szName && strlen (szName)) {
     menuitem = gtk_menu_item_new_with_label (szName);
-    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-                        GTK_SIGNAL_FUNC (func), data);
+    if(func)
+      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+			  GTK_SIGNAL_FUNC (func), data);
 
     GGobi_widget_set (GTK_WIDGET (menuitem), gg,  true);
 
@@ -237,8 +238,10 @@ submenu_make (gchar *lbl, guint key, GtkAccelGroup *accel_group) {
   tmp_key = gtk_label_parse_uline (GTK_LABEL (GTK_BIN (item)->child), lbl);
 
   /*-- V -> ViewMode works but O -> Options doesn't --*/
+#ifndef GTK_2_0
   gtk_widget_add_accelerator (item,
     "activate_item", accel_group, tmp_key, GDK_MOD1_MASK, GTK_ACCEL_LOCKED);
+#endif
 
   gtk_widget_show (item);
   return item;
@@ -309,7 +312,16 @@ variable_notebook_subwindow_add (datad *d, GtkSignalFunc func,
   gchar *row[1];
   vartabled *vt;
   GtkSelectionMode mode = (GtkSelectionMode)
-    gtk_object_get_data (GTK_OBJECT(notebook), "SELECTION");
+          gtk_object_get_data(GTK_OBJECT(notebook), "SELECTION");
+
+#ifdef GTK_2_0
+/* It appears (simple test) that the default mode is GTK_SELECTION_NONE. 
+   And then calling gtk_clist_set_selection_mode will cause the
+   default: case to assert a failure.
+   In Gtk 1.2, the default value is GTK_SELECTION_SINGLE (on my machines). */
+  if(mode == GTK_SELECTION_NONE)
+     mode = GTK_SELECTION_SINGLE;
+#endif
 
   if (d->ncols == 0)
     return;
@@ -347,8 +359,8 @@ variable_notebook_subwindow_add (datad *d, GtkSignalFunc func,
   clist = gtk_clist_new (1);
   gtk_clist_set_selection_mode (GTK_CLIST (clist), mode);
   gtk_object_set_data (GTK_OBJECT (clist), "datad", d);
-  gtk_signal_connect (GTK_OBJECT (clist), "select_row",
-    GTK_SIGNAL_FUNC (func), gg);
+  if(func)
+     gtk_signal_connect (GTK_OBJECT (clist), "select_row", GTK_SIGNAL_FUNC (func), gg);
 
   for (j=0; j<d->ncols; j++) {
     vt = vartable_element_get (j, d);
@@ -357,6 +369,10 @@ variable_notebook_subwindow_add (datad *d, GtkSignalFunc func,
         (vtype == integer && vt->vartype == integer) ||
         (vtype == real && vt->vartype == real))
     {
+      /*
+       * I can't use collab_tform, because then I can't find
+       * the variable indices later using only the name.
+      */
       row[0] = g_strdup (vt->collab);
       gtk_clist_append (GTK_CLIST (clist), row);
       g_free (row[0]);
@@ -486,6 +502,10 @@ variable_notebook_varchange_cb (ggobid *gg, vartabled *vt, gint which,
     for (j=0; j<d->ncols; j++) {
       vt = vartable_element_get (j, d);
       if (vt) {
+        /*
+         * I can't use collab_tform, because then I can't find
+         * the variable indices later using only the name.
+        */
         row[0] = g_strdup_printf (vt->collab);
         gtk_clist_append (GTK_CLIST (clist), row);
       }
@@ -608,3 +628,4 @@ GGobi_addToolsMenuItem (gchar *lbl, ggobid *gg)
 
   return (entry);
 }
+
