@@ -23,13 +23,20 @@ static void varcircle_pack (GtkWidget *, datad *);
 void
 varcircles_visibility_set (displayd *display, ggobid *gg)
 {
-  gint projection = (gint) projection_get (gg);
+  gint projection;
   gint j;
   GtkWidget *box;
-  datad *d = display->d;
-  GList *children = gtk_container_children (GTK_CONTAINER (d->vcirc_ui.table));
+  datad *d;
+  GList *children;
   gint n = 0;
 
+  if(!display)
+      return;
+
+  projection = (gint) projection_get (gg);
+  d = display->d;
+  children = gtk_container_children (GTK_CONTAINER (d->vcirc_ui.table));
+
   switch (projection) {
 
     case TOUR2D3:
@@ -71,7 +78,6 @@ varcircles_visibility_set (displayd *display, ggobid *gg)
             gtk_box_reorder_child (GTK_BOX (d->vcirc_ui.table), box, n);
             gtk_widget_show_all (box);
 #if GTK_MAJOR_VERSION == 1
-/*g_printerr ("ref_count: %d\n", GTK_OBJECT (box)->ref_count); */
             if (GTK_OBJECT (box)->ref_count > 1)
 #else
             if (G_OBJECT (box)->ref_count > 1)
@@ -225,7 +231,7 @@ manip_select_cb (GtkWidget *w, GdkEvent *event, datad *d)
 
   d->vcirc_ui.cursor = gdk_cursor_new (GDK_HAND2);
   gdk_window_set_cursor (window, d->vcirc_ui.cursor);
-  d->vcirc_ui.jcursor = (gint )GDK_HAND2;
+  d->vcirc_ui.jcursor = (gint)GDK_HAND2;
   
   return true;
 }
@@ -279,6 +285,9 @@ da_freeze_expose_cb (GtkWidget *w, GdkEvent *event, datad *d)
 void
 varcircles_show (gboolean show, datad *d, displayd *display, ggobid *gg)
 {
+  GtkWidget *basement = widget_find_by_name (gg->main_window, "BASEMENT");
+  GtkWidget *parent = (d->vcirc_ui.ebox)->parent;
+
   if (show) {
     /*
      * Add the ebox for the table of variable circles/rectangles
@@ -286,37 +295,53 @@ varcircles_show (gboolean show, datad *d, displayd *display, ggobid *gg)
     */
     if (display)
       varcircles_visibility_set (display, gg);
-    gtk_paned_pack2 (GTK_PANED (d->varpanel_ui.hpane),
-      d->vcirc_ui.ebox, true, true);
-    gtk_paned_set_handle_size (GTK_PANED(d->varpanel_ui.hpane), 10);
-    gtk_paned_set_gutter_size (GTK_PANED(d->varpanel_ui.hpane), 15);
 
-    /*-- update the reference count for the ebox --*/
-#if GTK_MAJOR_VERSION == 1
-    if (GTK_OBJECT (d->vcirc_ui.ebox)->ref_count > 1)
-#else
-    if (G_OBJECT (d->vcirc_ui.ebox)->ref_count > 1)
+    /* reparent the variable circles */
+    if (parent == basement) {
+      gtk_widget_ref (d->vcirc_ui.ebox);
+      gtk_container_remove (GTK_CONTAINER(basement), d->vcirc_ui.ebox);
+      gtk_paned_pack2 (GTK_PANED (d->varpanel_ui.hpane),
+        d->vcirc_ui.ebox, true, true);
+      gtk_widget_show (d->vcirc_ui.ebox);
+    } else if (parent == NULL) {
+      gtk_paned_pack2 (GTK_PANED (d->varpanel_ui.hpane),
+        d->vcirc_ui.ebox, true, true);
+    }
+
+#ifndef GTK_2_0
+    gtk_paned_set_handle_size (GTK_PANED(d->varpanel_ui.hpane), 10);
 #endif
-      gtk_widget_unref (d->vcirc_ui.ebox);
+    gtk_paned_set_gutter_size (GTK_PANED(d->varpanel_ui.hpane), 15);
 
   } else {
     /*-- remove circles/rectangles --*/
-    gtk_widget_ref (d->vcirc_ui.ebox);
-    gtk_container_remove (GTK_CONTAINER (d->varpanel_ui.hpane),
-                                         d->vcirc_ui.ebox);
+
+    if (parent == d->varpanel_ui.hpane) {
+      gtk_widget_hide (d->vcirc_ui.ebox);
+      gtk_widget_ref (d->vcirc_ui.ebox);
+      gtk_container_remove (GTK_CONTAINER (d->varpanel_ui.hpane),
+                                           d->vcirc_ui.ebox);
+      gtk_box_pack_start (GTK_BOX(basement), d->vcirc_ui.ebox, false, false, 0);
+    }
+
+
+#ifndef GTK_2_0
     gtk_paned_set_handle_size (GTK_PANED(d->varpanel_ui.hpane), 0);
+#endif
     gtk_paned_set_gutter_size (GTK_PANED(d->varpanel_ui.hpane), 0);
     /*-- set the handle position all the way to the right --*/
     gtk_paned_set_position (GTK_PANED(d->varpanel_ui.hpane), -1);
 
 
     /*-- adjust the reference count --*/
+    /*
 #if GTK_MAJOR_VERSION == 1
     if (GTK_OBJECT (d->vcbox_ui.ebox)->ref_count > 1)
 #else
     if (G_OBJECT (d->vcbox_ui.ebox)->ref_count > 1)
 #endif
-      gtk_widget_unref (d->vcbox_ui.ebox);
+        gtk_widget_unref (d->vcbox_ui.ebox);
+    */
   }
 }
 
@@ -642,7 +667,7 @@ varcircle_draw (gint jvar, datad *d, ggobid *gg)
   /*-- add the appropriate line --*/
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
     GtkGGobiExtendedDisplayClass *klass;
-    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+    klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
     if(klass->varcircle_draw)
       chosen = klass->varcircle_draw(display, jvar, da_pix, gg);
   }
@@ -714,4 +739,3 @@ varcircles_add (gint nc, datad *d, ggobid *gg)
 
   gtk_widget_show_all (gg->varpanel_ui.notebook);
 }
-
