@@ -43,6 +43,9 @@ varpanel_widget_get_nth (gint jbutton, gint jvar, datad *d)
 {
   GtkWidget *box, *child;
   box = (GtkWidget *) varpanel_container_get_nth (jvar, d);
+  if(!box)
+    return(NULL);
+
   child = (GtkWidget *) gtk_object_get_data (GTK_OBJECT (box),
     varpanel_names[jbutton]);
   return child;
@@ -53,8 +56,15 @@ varpanel_label_set (gint j, datad *d)
   GtkWidget *label = varpanel_widget_get_nth (VARSEL_LABEL, j, d);
   vartabled *vt = vartable_element_get (j, d);
   /*-- the label is actually a button; this is the label --*/
-  GtkWidget *labelw = GTK_BIN(label)->child;
+  GtkWidget *labelw;
 
+  if(!label || !GTK_IS_BIN(label))
+    return;
+
+  labelw = GTK_BIN(label)->child;
+
+  if(!labelw)
+    return;
   /*-- make sure it stays left-aligned --*/
   gtk_misc_set_alignment (GTK_MISC(labelw), 0, .5);
   gtk_label_set_text (GTK_LABEL(labelw), vt->collab_tform);
@@ -130,11 +140,20 @@ varsel (GtkWidget *w, cpaneld *cpanel, splotd *sp, gint jvar,
   }
 
   if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
-     redraw = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass)->variable_select(w, display, sp, jvar, toggle, mousebtn, cpanel, gg);
+     redraw = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display))->variable_select(w, display, sp, jvar, toggle, mousebtn, cpanel, gg);
   }
 
-  gtk_signal_emit(GTK_OBJECT(gg), GGobiSignals[VARIABLE_SELECTION_SIGNAL], 
-    display->d, jvar, sp);
+  /* 
+   * I'm not sure what we use this for, but it causes a problem for
+   * the composite displays when a plot is deleted.  For example, when
+   * a plot is deleted in the parallel coordinates plot, error
+   * messages are generated by this signal_emit, presumably because sp
+   * is NULL for some subroutine called by this guy.  If sp has to be
+   * the current splot, that case is suppressed.
+   */
+  if (sp == gg->current_splot)
+    gtk_signal_emit(GTK_OBJECT(gg), GGobiSignals[VARIABLE_SELECTION_SIGNAL], 
+      display->d, jvar, sp);
 
   /*-- overkill for scatmat: could redraw one row, one column --*/
   /*-- overkill for parcoords: need to redraw at most 3 plots --*/
@@ -198,8 +217,8 @@ varpanel_switch_page_cb (GtkNotebook *notebook, GtkNotebookPage *page,
   if (gg->status_message_func) {
     datad *d = (datad *) g_slist_nth_data (gg->d, page_num);
     if (d) {
-      gchar *msg = g_strdup_printf ("%s: %d x %d\n",
-        d->name, d->nrows, d->ncols);
+      gchar *msg = g_strdup_printf ("%s: %d x %d (%s)",
+        d->name, d->nrows, d->ncols, gg->input->fileName);
       gg->status_message_func(msg, gg);
       g_free (msg);
     }
@@ -219,7 +238,7 @@ varpanel_refresh (displayd *display, ggobid *gg)
 
     if (sp != NULL && d != NULL) {
       if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
-        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass)->varpanel_refresh(display, sp, d);
+        GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display))->varpanel_refresh(display, sp, d);
       }
     }
   } else {
@@ -390,6 +409,13 @@ varpanel_addvar_cb (ggobid *gg, vartabled *vt, gint which,
   varcircles_add (d->ncols, d, gg);
 
   /*-- make sure the right toggle widgets and circles are showing --*/
+/* this gives the wrong result when the variable being added is
+not displayed in the current display.  And I don't know what the mode
+is for the other datad.  This suggests that I have to run these two
+routines whenever the tab changes in the variable selection panel.
+But even then it may not correspond to the current display.  So it
+really suggests that the variable selection panel has to keep track of
+its own projection.  Add a variable to varpanel_ui in datad.h? -- dfs */
   varpanel_refresh (gg->current_display, gg);
   varcircles_visibility_set (gg->current_display, gg);
 }
@@ -443,7 +469,8 @@ varpanel_clear (datad *d, ggobid *gg)
 
 
 /*-- for each datad, a scrolled window, vbox, hbox, togglebuttons and label --*/
-void varpanel_populate (datad *d, ggobid *gg)
+void 
+varpanel_populate (datad *d, ggobid *gg)
 {
   gint j, nd;
 
@@ -456,18 +483,24 @@ void varpanel_populate (datad *d, ggobid *gg)
 
   /*-- create a paned widget --*/
   d->varpanel_ui.hpane = gtk_hpaned_new ();
+#ifndef GTK_2_0
   gtk_paned_set_handle_size (GTK_PANED(d->varpanel_ui.hpane), 0);
+#endif
   gtk_paned_set_gutter_size (GTK_PANED(d->varpanel_ui.hpane), 0);
   /*-- set the handle position all the way to the right --*/
   gtk_paned_set_position (GTK_PANED(d->varpanel_ui.hpane), -1);
 
   gtk_object_set_data(GTK_OBJECT(d->varpanel_ui.hpane), "datad", d);/*setdata*/
   /*-- only add a tab if there are variables --*/
-  if (g_slist_length (d->vartable) > 0) {
+  if (g_slist_length (d->vartable) > 0 || d->ncols > 0) {
     gtk_notebook_append_page (GTK_NOTEBOOK (gg->varpanel_ui.notebook),
-      d->varpanel_ui.hpane, gtk_label_new (d->name));
+			      d->varpanel_ui.hpane, gtk_label_new (d->name));
   }
 
+  /* Check if we have been here before and already created the box, etc.. */
+  if(d->vcbox_ui.ebox && GTK_IS_WIDGET(d->vcbox_ui.ebox))
+    return;
+
   /*-- create an ebox, and put it in the hpane --*/
   d->vcbox_ui.ebox = gtk_event_box_new ();
   gtk_paned_pack1 (GTK_PANED(d->varpanel_ui.hpane),
@@ -506,7 +539,7 @@ GGOBI(selectScatterplotX) (GtkWidget *w, gint jvar, ggobid *gg)
 
   if(!GTK_IS_GGOBI_EXTENDED_DISPLAY(display))
     return;
-  klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+  klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
   if(klass->select_X)
     klass->select_X(w, display, jvar, gg);
 }
@@ -551,7 +584,7 @@ varpanel_tooltips_set (displayd *display, ggobid *gg)
       label = varpanel_widget_get_nth (VARSEL_LABEL, j, d);
     
       if(GTK_IS_GGOBI_EXTENDED_DISPLAY(display)) {
-         GtkGGobiExtendedDisplayClass *klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT(display)->klass);
+         GtkGGobiExtendedDisplayClass *klass = GTK_GGOBI_EXTENDED_DISPLAY_CLASS(GTK_OBJECT_GET_CLASS(display));
          if(klass->varpanel_tooltips_set)
            klass->varpanel_tooltips_set(display, gg, wx, wy, wz, label);
       }
