@@ -390,7 +390,10 @@ dialog_newvar_add (GtkWidget *w, ggobid *gg)
   }
   vname = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
   if (vname != NULL && strlen(vname) > 0) {
-    newvar_add (vtype, vname, d, gg);
+     newvar_add_with_values(vtype == ADDVAR_BGROUP ? 
+       (gdouble *) &AddVarBrushGroup : (gdouble *) &AddVarRowNumbers,
+       d->nrows, vname, real,
+       0, NULL, NULL, NULL, d, gg); 
 
 /* I think we still want to do this ... */
 #ifdef FORMERLY
@@ -507,6 +510,7 @@ dialog_rename_var (GtkWidget *w, ggobid *gg)
   if (vname != NULL && strlen(vname) > 1) {
     vt = vartable_element_get (jvar, d);
     vt->collab = g_strdup (vname);
+    vt->nickname = g_strndup (vname, 2);
 
     vartable_collab_set_by_var (jvar, d);
     tform_label_update (jvar, d, gg);
