@@ -158,18 +158,21 @@ colorscheme_set_cb (GtkWidget *w, colorschemed* scheme)
  * and there's probably a way to do it better.
 */
   clist = get_clist_from_object (GTK_OBJECT (w));
-  if(clist == NULL) {
+  if(clist != NULL) {
       d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
       selected_var = get_one_selection_from_clist (clist, d);
   } else {
       d = (datad *) g_slist_nth_data(gg->d, 0);
       selected_var = 0;
   }
+
   if (d && selected_var != -1) {
       gtk_signal_emit_by_name (GTK_OBJECT (gg->wvis.da), "expose_event",
         (gpointer) gg, (gpointer) &rval);
+
+      bin_counts_reset (selected_var, d, gg);
   }
-  bin_counts_reset (selected_var, d, gg);
+
   gtk_signal_emit_by_name (GTK_OBJECT (gg->wvis.da), "expose_event",
     (gpointer) gg, (gpointer) &rval);
 }
@@ -274,12 +277,17 @@ motion_notify_cb (GtkWidget *w, GdkEventMotion *event, ggobid *gg)
   gfloat val;
 
   GtkWidget *clist = get_clist_from_object (GTK_OBJECT (w));
-  datad *d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
-  gint selected_var = get_one_selection_from_clist (clist, d);
+  datad *d = NULL;
+  gint selected_var = -1;
 
   icoords *mousepos = &gg->wvis.mousepos;
   gint color = gg->wvis.nearest_color;
 
+  if(clist) {
+    d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
+    selected_var = get_one_selection_from_clist (clist, d);
+  }
+
   gdk_window_get_pointer (w->window, &pos.x, &pos.y, &state);
 
   if (pos.x != mousepos->x) {
@@ -360,8 +368,13 @@ static gint
 button_release_cb (GtkWidget *w, GdkEventButton *event, ggobid *gg)
 {
   GtkWidget *clist = get_clist_from_object (GTK_OBJECT (w));
-  datad *d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
-  gint selected_var = get_one_selection_from_clist (clist, d);
+  datad *d = NULL; 
+  gint selected_var = -1;
+
+  if(clist) {
+      d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
+      selected_var = get_one_selection_from_clist (clist, d);
+  }
 
   if (gg->wvis.motion_notify_id) {
     gtk_signal_disconnect (GTK_OBJECT (w), gg->wvis.motion_notify_id);
@@ -495,12 +508,17 @@ da_expose_cb (GtkWidget *w, GdkEventExpose *event, ggobid *gg)
     gg->wvis.scheme : gg->activeColorScheme;
 
   GtkWidget *clist = get_clist_from_object (GTK_OBJECT (w));
-  datad *d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
-  gint selected_var = get_one_selection_from_clist (clist, d);
+  datad *d = NULL;
+  gint selected_var = -1;
 
   GtkWidget *da = gg->wvis.da;
   GdkPixmap *pix = gg->wvis.pix;
 
+  if(clist) {
+   d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
+   selected_var = get_one_selection_from_clist (clist, d);
+  }
+
   if (gg->wvis.GC == NULL)
     gg->wvis.GC = gdk_gc_new (w->window);
 
@@ -545,7 +563,7 @@ da_expose_cb (GtkWidget *w, GdkEventExpose *event, ggobid *gg)
   y = ymargin + 10;
   for (k=0; k<scheme->n-1; k++) {
     x = xmargin + gg->wvis.pct[k] * (w->allocation.width - 2*xmargin);
-    draw_3drectangle (pix, x, y, 20, 10, gg);
+    draw_3drectangle (w, pix, x, y, 20, 10, gg);
     y += hgt;
   }
 
@@ -740,9 +758,12 @@ gboolean colors_remap (colorschemed *scheme, gboolean force, ggobid *gg)
 static void scale_set_cb (GtkWidget *w, ggobid* gg)
 {
   GtkWidget *clist = get_clist_from_object (GTK_OBJECT (w));
-  datad *d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
+  datad *d = NULL;
   gboolean rval = false;
 
+  if(clist)
+    d = (datad *) gtk_object_get_data (GTK_OBJECT (clist), "datad");
+
   /*
    * If we've been using gg->wvis.scheme, set gg->activeColorScheme
    * to the current scheme.
@@ -816,6 +837,18 @@ entry_set_scheme_name (ggobid *gg)
                                 gg->activeColorScheme->name);
 }
 
+/* Can close the window via an event or in the ggobi_close routine.
+   The former (i.e. events) are more extensible and localized. */
+void
+close_wvis_window_cb(GtkWidget *w, GdkEventButton *event, ggobid *gg)
+{
+fprintf(stderr, "Closing the color scheme window\n");fflush(stderr);
+#if 0
+   gtk_widget_destroy(gg->wvis.window);
+   gg->wvis.window = NULL;
+#endif
+}
+
 void
 wvis_window_open (ggobid *gg) 
 {
@@ -839,8 +872,10 @@ wvis_window_open (ggobid *gg)
     "Update on mouse release",
     "Update continuously"};
 
+#if 0
   if (gg->d == NULL || g_slist_length (gg->d) == 0)
     return;
+#endif
 
   if (gg->wvis.window == NULL) {
 
@@ -850,6 +885,18 @@ wvis_window_open (ggobid *gg)
     gtk_signal_connect (GTK_OBJECT (gg->wvis.window),
       "delete_event", GTK_SIGNAL_FUNC (close_wmgr_cb), gg);
 
+#if 0 /* Testing */
+    gtk_signal_connect (GTK_OBJECT (gg->main_window),
+                        "delete_event",
+                        GTK_SIGNAL_FUNC (close_wvis_window_cb),
+                        (gpointer) gg);
+
+    gtk_signal_connect (GTK_OBJECT (gg->main_window),
+                        "destroy",
+                        GTK_SIGNAL_FUNC (close_wvis_window_cb),
+                        (gpointer) gg);
+#endif
+
     hpane = gtk_hpaned_new();
     gtk_container_add (GTK_CONTAINER (gg->wvis.window), hpane);
 
